# coding:utf-8
from ...backend import db


def is_logined(req, session):
    user_id = session.get('user_id')
    print(user_id, session)

    if not user_id:
        return False
    if getattr(req, 'user', None)\
            and str(req.user.id) == user_id:
        return True
    else:
        user = db.get_inst(db.User, id=user_id, soft_del=False)
        if user:
            req.user = user
            return True
    return False


def login(user, req, session):
    if user and user.soft_del != True:
        session['user_id'] = str(user.id)
        req.user = user
        return True
    return False


def logout(req, session):
    req.user = None
    session.clear()


def record_login(user_id, login_type, sid=None, **info):
    record = db.LoginRecord(user_id=user_id,
                            sid=sid,
                            login_type=login_type,
                            info=info)
    record.save()
    return record


def get_record_desc(record):
    ret = {'time': record.created.isoformat(),
           'record_id': str(record.id),
           'user_id': record.user_id,
           'login_type': record.login_type}
    return ret


def get_user_login_record(user_id, **kw):
    desc = kw.get('desc', True)
    cursor_id = kw.get('cursor_id') or None
    limit = kw.get('limit', 10)
    filters = {'user_id': user_id}
    if cursor_id:
        filters['$lt'] = ['id', cursor_id]
    log_objs = list(db.get_list_query(db.LoginRecord,
                                      order_by='-id',
                                      **filters)\
                    .limit(limit))
    ret = [get_record_desc(i) for i in log_objs]
    return True, ret



# coding:utf-8
from collections import deque
from ...util.cache_util import cache_lock
from ...util.chinese import chinese_num_replace
from . import user_perm as user_perm_ops
from ...backend import db


def position_to_dict(position):
    ret = {'id': str(position.id),
           'name': position.name,
           'foreign_key': foreign_key,
           'tags': tags,
           'info': info,
           'parent_id': position.parent_id}
    return ret


def get_position_info(position_id):
    position = db.get_inst(db.PositionDesc,
                           id=position_id, soft_del=False)
    if not position:
        return None
    info = position_to_dict(position)
    if position.path:
        root_position = db.get_inst(db.PositionDesc,
                                    id=position.path[0],
                                    soft_del=False)
        if root_position:
            info['root_name'] = root_position.name
    return info


def get_sub_positions(position_ids, included=False):
    '''
    获取某些职位(id)的全部下级职位。
    Params
    ------
    - position_ids: 职位id列表
    - included: boolean, default False, 是否包含position_ids中的职位
    '''
    filters = {'soft_del': False}
    if included:
        filters.update({'$in': ['id', position_ids],
                        '$in': ['path', position_ids]})
    else:
        filters.update({'$in': ['path', position_ids]})
    positions = list(db.get_list_query(db.PositionDesc, **filters))
    positions = sorted(positions, key=lambda x: chinese_num_replace(x.name))
    return positions


def get_granted_position_ids(user_id, include_roles=True):
    '''
    get_granted_position_ids(user_id, include_roles=True)
    获取授权给用户的position_id列表，这个列表是未经扩展过的
    '''
    position_ids = set()
    user_perm = db.get_inst(db.UserPerm,
                            user_id=user_id, soft_del=False)
    if not user_perm:
        pass
    else:
        position_ids |= set(user_perm.granted_positions)
        if include_roles and user_perm.roles:
            roles = db.get_list_query(db.RoleDesc,
                                      **{'$in': ['id', user_perm.roles],
                                         'soft_del': False})
            for role in roles:
                position_ids |= set(role.granted_positions)
    return position_ids


def get_all_granted_position_ids(user_id):
    position_ids = get_granted_position_ids(user_id)
    sub_positions = get_sub_positions(position_ids, included=True)
    return [str(i.id) for i in sub_positions]


def get_users_by_positions(position_ids,
                           include_deleted_users=False):
    user_perms = db.get_list_query(db.UserPerm,
                                   **{'$in': [position_id, position_ids],
                                      'soft_del': False})
    user_ids = map(lambda x: x.user_id, user_perms)
    filters = {'$in': ['id', user_ids]}
    if not include_deleted_users:
        filters['soft_del'] = False
    users = db.get_list_query(db.User, **filters)
    return list(users)


def get_users_by_sub_positions(position_ids,
                               included=True,
                               include_deleted_users=False):
    sub_position_ids = map(lambda x: str(x.id),
                           get_sub_positions(position_ids,
                                             included=included))
    return get_users_by_positions(sub_position_ids,
                                  include_deleted_users=include_deleted_users)


def get_all_sub_users(user_id):
    position_ids = get_granted_position_ids(user_id)
    sub_positions = get_sub_positions(position_ids, included=True)
    users = []
    if not sub_positions:
        pass
    else:
        users = get_users_by_positions(position_ids)
    user_perm = db.get_inst(db.UserPerm, soft_del=False, user_id=user_id)
    if user_perm and user_perm.position_id:
        user = db.get_inst(db.User, id=user_id)
        if user:
            users.append(user)
    return users


def is_upstream_position(curr_user, position_id=None, user_id=None, position_ids=None):
    curr_user = curr_user if hasattr(curr_user, 'id')\
                else db.get_inst(db.User, id=curr_user)
    if position_ids:
        pass
    elif user_id and not position_id:
        user_perm = db.get_inst(db.UserPerm, user_id=user_id, soft_del=False)
        if user_perm and user_perm.position_id:
            position_id = user_perm.position_id
            position_ids = [position_id]
    elif position_id:
        position_ids = [position_id]

    if not position_ids:
        return False
    position_ids = [i for i in position_ids if i]
    curr_position_ids = get_granted_position_ids(str(curr_user.id))
    if curr_position_ids:
        positions = {str(p.id): p for p in\
                     db.get_list_query(db.PositionDesc,
                                       **{'$in': ['id', position_ids],
                                          'soft_del': False})}
        if len(positions) != len(position_ids):
            return False
        for position_id, position in positions.items():
            if not any([i in position.path or i == position_id\
                        for i in curr_position_ids]):
                return False
        else:
            return True
    return False


def is_owner(curr_user, position_id=None, user_id=None):
    curr_user = curr_user if hasattr(curr_user, 'id')\
                else db.get_inst(db.User, id=curr_user)
    if user_id and str(curr_user.id) == user_id:
        return True
    #if not position_id:
    #    return False
    #curr_user_perm = UserPerm.objects(user_id=str(curr_user.id), soft_del=False).first()
    #if curr_user_perm and curr_user_perm.position_id == position_id:
    #    return True
    return False


def set_position(user_id, position_id):
    '''
    设置用户的职位，当position is None时，用户的职位为空。
    '''
    with cache_lock("user_%s" % user_id):
        position = None
        user_perm = user_perm_ops.get_or_create_user_perm(user_id)
        user = db.get_inst(db.User, id=user_id, soft_del=False)
        if not (user_perm and user):
            return False, "用户不存在或已删除"
        position_id = position_id or ''
        if position_id:
            position = db.get_inst(db.PositionDesc,
                                   id=position_id, soft_del=False)
            if not position:
                return False, "职位不存在"
        user_perm.position_id = position_id
        user_perm.save()
        user.info['position_id'] = position_id
        user.info['position_name'] = position.name if position else ""
        user.info['position_tags'] = position.tags if position else []
        user.info['position_path'] = position.path if position else []
        user.save()
        return True, user_perm


def set_granted_positions(user_id, position_ids):
    user = db.get_inst(db.User, id=user_id, soft_del=False)
    if not user:
        return False, "用户不存在或已删除"
    user_perm = user_perm_ops.get_or_create_user_perm(user_id)
    positions = db.get_list_query(db.PositionDesc,
                                  **{'soft_del': False,
                                     '$in': ['id', position_ids]})
    position_ids = list(map(lambda x: str(x.id), positions))
    user_perm.granted_positions = position_ids
    user_perm.save()
    return True, user_perm


def grant_position(op, position_id, user_id=None, role_id=None):
    if not user_id and not role_id:
        return False, "未提供user_id或者role_id"
    position = db.get_inst(db.PositionDesc,
                           soft_del=False, id=position_id)
    if not position:
        return False, "职位或团队id未提供"
    perm_obj = None
    if user_id:
        perm_obj = user_perm_ops.get_or_create_user_perm(user_id)
    elif role_id:
        perm_obj = db.get_inst(db.RoleDesc,
                               soft_del=False, id=role_id)
    if not perm_obj:
        return False, "目标用户或者角色不存在"
    if op == 'add':
        perm_obj.granted_positions = list(set(perm_obj.granted_positions) | set([position_id]))
    elif op == 'del':
        perm_obj.granted_positions = list(filter(lambda x: x != position_id, perm_obj.granted_positions))
    else:
        raise Exception("Unknown exception names.")
    perm_obj.save()
    return True, perm_obj.granted_positions


def position_id_by_name(name):
    position = db.get_list(db.PositionDesc,
                           soft_del=False, name=name)
    return str(position.id) if position else None


def position_ids_by_names(names):
    positions = db.get_list_query(db.PositionDesc,
                                  **{'soft_del': False,
                                     '$in': ['name', names]})
    return list(map(lambda x: str(x.id), positions))


def get_position_by_foreign_key(foreign_key):
    if not foreign_key:
        return None
    position = db.get_inst(db.PositionDesc,
                           foreign_key=foreign_key,
                           soft_del=False)
    return position


def add_position(name,
                 parent_id=None,
                 parent_name=None,
                 foreign_key=None,
                 allow_duplicate=False):
    cache_key = "position"
    if parent_name and not parent_id and not allow_duplicate:
        parent_position = db.get_inst(db.PositionDesc,
                                      soft_del=False,
                                      name=parent_name)
        if parent_position:
            parent_id = str(parent_position.id)
    with cache_lock(cache_key):
        position = get_position_by_foreign_key(foreign_key)
        if position:
            return False, "外键已经存在"
        position = db.get_inst(db.PositionDesc,
                               soft_del=False, name=name)
        if allow_duplicate is not True and position:
            return False, "职位或团队名称重复"
        parent = None
        if parent_id:
            parent = db.get_inst(PositionDesc,
                                 soft_del=False, id=parent_id)
            if not parent:
                return False, "职位或团队的上级职位不存在"
        position = db.PositionDesc(name=name,
                                   parent_id=parent_id,
                                   foreign_key=foreign_key)
        if parent:
            position.path = parent.path + [parent_id]
        position.save()
        return True, position


def remove_position(position_id, allow_recursive=False):
    cache_key = "position"
    with cache_lock(cache_key):
        if not allow_recursive and\
                db.get_inst(db.PositionDesc,
                            soft_del=False, parent_id=position_id):
            return False, "目标职位或团队尚有下级存在"
        if allow_recursive:
            for i in db.get_list_query(db.PositionDesc,
                                       soft_del=False, path=position_id):
                i.soft_del = True
                i.save()
        position = db.get_inst(db.PositionDesc,
                               id=position_id)
        if position and not position.soft_del:
            position.soft_del = True
            position.save()
        return True, position


def move_position(position_id, target_parent_id=''):
    cache_key = "position"
    with cache_lock(cache_key):
        position = db.get_inst(db.PositionDesc,
                               soft_del=False, id=position_id)
        if not position:
            return False, "职位或团队不存在"
        target_parent = None
        if target_parent_id:
            target_parent = db.get_inst(db.PositionDesc,
                                        soft_del=False, id=target_parent_id)
            if not target_parent:
                return False, "上级职位或团队不存在"
            if str(position.id) in target_parent.path:
                return False, "上级职位是当前职位的下级"
            if position_id == target_parent_id:
                return False, "上级职位于当前职位相同"
        position.parent_id = target_parent_id
        if target_parent:
            position.path = target_parent.path + [target_parent_id]
        else:
            position.path = []
        position.save()
        for i in db.get_list_query(db.PositionDesc,
                                   soft_del=False, path=position_id):
            pos = i.path.index(position_id)
            i.path = position.path + i.path[pos:]
            i.save()
        return True, position


def rename_position(position_id, new_name, allow_duplicate=False):
    cache_key = "position"
    with cache_lock(cache_key):
        if not allow_duplicate and db.get_inst(db.PositionDesc,
                                               soft_del=False,
                                               name=new_name):
            return False, "相同名称的职位或者团队已经存在"
        position = db.get_inst(db.PositionDesc, soft_del=False, id=position_id)
        if not position:
            return False, "职位或者团队不存在"
        position.name = new_name
        position.save()
        return True, position


def position_tree(position_ids,
                  extend_users=False,
                  extra_fields=None):
    '''
    获取职位的树形结构
    '''
    def convert(obj):
        ret = {'label': obj.name,
               'value': str(obj.id),
               'children': []}
        for field in extra_fields:
            ret[field] = getattr(obj, field, None)
        return ret
    if extra_fields is None:
        extra_fields = []
    positions = get_sub_positions(position_ids, included=True)
    all_ids = set(map(lambda x: str(x.id), positions))

    users_dict = {}
    if extend_users:
        user_perm_list = list(db.get_list_query(db.UserPerm,
                                                **{'soft_del': False,
                                                   '$in': ['position_id', all_ids]}))
        users = {str(u.id): {'label': u.name, 'value': 'user %s' % str(u.id)}\
                     for u in db.get_list_query(db.User,
                                                **{'soft_del': False,
                                                   '$in': ['id', list(map(lambda x: x.user_id, user_perm_list))]})}
        for up in user_perm_list:
            users_dict.setdefault(up.position_id, [])
            user = users.get(up.user_id)
            if user:
                users_dict[up.position_id].append(user)

    head_positions = filter(lambda x: x.parent_id not in all_ids,
                            positions)
    stack = deque()
    stack.extend(list(map(convert, head_positions)))
    ret = [i for i in stack]
    while stack:
        curr = stack.popleft()
        subs = list(map(convert,
                        filter(lambda x: x.parent_id == curr['value'],
                               positions)))
        curr['children'] = []
        curr['children'].extend(subs)
        if extend_users:
            curr['children'].extend(users_dict.get(curr['value']) or [])
        if not curr['children']:
            del curr['children']
        else:
            curr['children'] = [{'label': '全部', 'value': curr['value']}]\
                               + curr['children']
        stack.extendleft(subs)
    return ret


def position_tree_dump():
    ret = []
    positions = list(db.get_list_query(db.PositionDesc, soft_del=False))
    stack = deque()
    stack.extend(list(filter(lambda x: not x.parent_id, positions)))
    while stack:
        curr = stack.popleft()
        subs = (list(filter(lambda x: x.parent_id == str(curr.id),
                            positions)))
        stack.extendleft(subs)
        output = "%s %s %s %s" % ("    " * len(curr.path or []), "+" if subs else "-", curr.name, str(curr.id))
        ret.append(output)
    return '\n'.join(ret)


def fix_path():
    positions = list(db.get_list_query(db.PositionDesc,
                                       soft_del=False))
    stack = deque()
    stack.extend(list(filter(lambda x: not x.parent_id, positions)))
    while stack:
        curr = stack.popleft()
        if not curr.parent_id:
            curr.path = []
        else:
            parent = db.get_inst(db.PositionDesc, id=curr.parent_id)
            curr.path = parent.path + [curr.parent_id]
        curr.save()
        subs = (list(filter(lambda x: x.parent_id == str(curr.id),
                            positions)))
        stack.extendleft(subs)


def get_position_leaders(position_id):
    user_perms = db.get_list_query(db.UserPerm,
                                   position_id=position_id,
                                   granted_positions=position_id)
    user_ids = [up.user_id for up in user_perms]
    return user_ids



import logging
import datetime
import bson
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declarative_base
from ... import default

logger = logging.getLogger(__name__)


engine = None

session = None

Base = declarative_base()


def bson_id_gen():
    oid = str(bson.objectid.ObjectId())
    return oid


def model_save(self):
    try:
        session.add(self)
        session.commit()
    except Exception as e:
        session.close()

Base.save = model_save


class User(Base):
    __tablename__ = 'user'

    id = sa.Column(sa.String(length=32), default=bson_id_gen, primary_key=True)
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    name = sa.Column(sa.String(length=32, collation=default.MYSQL_COLLATION))
    state = sa.Column(sa.String(length=32, collation=default.MYSQL_COLLATION), server_default='normal')
    soft_del = sa.Column(sa.Boolean(),default=False, server_default=sa.text('0'))
    info = sa.Column(sa.JSON, default={})
    keywords = sa.Column(sa.String(length=4096))


class UserAuditLog(Base):
    __tablename__ = 'user_audit_log'

    id = sa.Column(sa.BIGINT, primary_key=True)
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    user_id = sa.Column(sa.String(length=32))
    collection_name = sa.Column(sa.String(length=32))
    editor_id = sa.Column(sa.String(length=32))
    editor_name = sa.Column(sa.String(length=32, collation=default.MYSQL_COLLATION))
    editor_info = sa.Column(sa.JSON, default={})
    fields = sa.Column(sa.JSON, default=[])
    delta = sa.Column(sa.JSON, default=[])


class PermDesc(Base):
    '''
    权限的描述，用于权限列表的展示，权限的提示等
    '''
    __tablename__ = 'perm_desc'

    id = sa.Column(sa.BIGINT, primary_key=True)
    action = sa.Column(sa.String(length=255))
    effect = sa.Column(sa.String(length=255), default='allow')
    resources = sa.Column(sa.JSON, default=[])
    name = sa.Column(sa.String(length=32))
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    soft_del = sa.Column(sa.Boolean(), default=False, server_default=sa.text('0'))
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)


class RoleDesc(Base):
    __tablename__ = 'role_desc'

    id = sa.Column(sa.BIGINT, primary_key=True)
    name = sa.Column(sa.String(length=32))
    perms = sa.Column(sa.JSON, default=[])
    granted_positions = sa.Column(sa.JSON, default=[])
    foreign_key = sa.Column(sa.String(length=255))
    info = sa.Column(sa.JSON, default=[])
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    soft_del = sa.Column(sa.Boolean(),default=False, server_default=sa.text('0'))
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)


class PositionDesc(Base):
    '''
    职位描述，每个用户通过关联职位描述来确定上下级关系和数据集权限
    --------
    name: string, 职位名称
    parent_id: string, 职位的父级职位id
    path: list, 从根节点到当前节点的id路径，包括根节点，不包括当前节点
    position_type: string, 职位的类型，用于区分一些特殊的职位，比如理财顾问
    '''
    __tablename__ = 'position_desc'

    id = sa.Column(sa.BIGINT ,primary_key=True)
    name = sa.Column(sa.String(length=32))
    parent_id = sa.Column(sa.BIGINT)
    path = sa.Column(sa.JSON, default=[])
    position_type = sa.Column(sa.String(length=32))
    foreign_key = sa.Column(sa.String(length=255))
    tags = sa.Column(sa.JSON, default=[])
    info = sa.Column(sa.JSON, default={})
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    soft_del = sa.Column(sa.Boolean(),default=False, server_default=sa.text('0'))
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)


class UserPerm(Base):
    __tablename__ = 'user_perm'

    id = sa.Column(sa.BIGINT, primary_key=True)
    user_id = sa.Column(sa.String(length=32))
    roles = sa.Column(sa.JSON, default=[])
    perms = sa.Column(sa.JSON, default=[])
    position_id = sa.Column(sa.BIGINT)
    granted_positions = sa.Column(sa.JSON, default=[])
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    soft_del = sa.Column(sa.Boolean(),default=False, server_default=sa.text('0'))
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)


class LoginRecord(Base):
    __tablename__ = 'login_record'

    id = sa.Column(sa.BIGINT, primary_key=True)
    user_id = sa.Column(sa.String(length=32))
    login_type = sa.Column(sa.String(length=32))
    sid = sa.Column(sa.String(length=255))
    expiration = sa.Column(sa.DateTime, default=datetime.datetime.now)
    info = sa.Column(sa.JSON, default={})
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)


class PasswordLogin(Base):
    __tablename__ = 'password_login'

    id = sa.Column(sa.BIGINT, primary_key=True)
    user_id = sa.Column(sa.String(length=32), nullable=False)
    username = sa.Column(sa.String(length=32), nullable=False)
    password = sa.Column(sa.String(length=32))
    soft_del = sa.Column(sa.Boolean(),default=False, server_default=sa.text('0'))
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)


class ResetPasswordRecord(Base):
    __tablename__ = 'reset_password_record'

    id = sa.Column(sa.BIGINT, primary_key=True)
    user_id = sa.Column(sa.String(length=32), nullable=False)
    code = sa.Column(sa.String(length=32), nullable=False)
    query_times = sa.Column(sa.BIGINT, default=0)
    state = sa.Column(sa.String(length=32), default='created')
    info = sa.Column(sa.JSON, default={})
    timestamp = sa.Column(sa.BIGINT, default=lambda: int(time.time()))
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)


class ThirdPartyLogin(Base):
    __tablename__ = 'third_party_login'

    id = sa.Column(sa.BIGINT, primary_key=True)
    login_type = sa.Column(sa.String(length=32), nullable=False)
    user_id = sa.Column(sa.String(length=32), nullable=False)
    ref_id = sa.Column(sa.String(length=255))
    info = sa.Column(sa.JSON, default={})
    soft_del = sa.Column(sa.Boolean(),default=False, server_default=sa.text('0'))
    created = sa.Column(sa.DateTime, default=datetime.datetime.now)
    lut = sa.Column(sa.DateTime, default=datetime.datetime.now)



# coding:utf-8
'''
Decorator for login and permission.
'''
import functools
import flask
from ...util import api_base
from ...auth.ops import common
from . import user_perm


def perm_required(perms=None):
    '''
    用于检查用户登录及基本的权限。
    '''
    if perms and not isinstance(perms, (tuple, list)):
        perms = [perms]
    def entangle(func):
        @functools.wraps(func)
        def wrapper(*sub, **kw):
            if not common.is_logined(flask.request, flask.session):
                return api_base.send_json_result("USER_NOT_LOGIN")
            if perms:
                for perm in perms:
                    if not user_perm.has_perm(flask.session['user_id'],
                                              perm):
                        return api_base.send_json_result("FORBIDDEN",
                                                         msg="缺少权限%s" % str(perm))
            ret = func(*sub, **kw)
            return ret
        return wrapper
    return entangle



# -*- coding:utf-8 -*-
'''
Basic root blueprint entry point.
'''
from .common import conf, pb_root
from .backend import init_db


def setup_conf(**config):
    '''
    write config to conf module
    '''
    for k, v in config.items():
        setattr(conf, k, v)


def setup_flask(flask_app, url_prefix="/pb"):
    '''
    Setup flask, register perm_broker as a blueprint.
    '''
    # import sub level path routings
    from .user.views import api as user_api
    from .auth.views import api as auth_api
    from .auth.views import web as auth_web
    from .perm.views import api as perm_api

    # enable debug login
    if conf.DEBUG and getattr(conf, 'ENABLE_DEBUG_LOGIN', False):
        from .auth.views import debug_api

    # bind perm broker blueprint
    flask_app.register_blueprint(pb_root, url_prefix=url_prefix)


def setup_perm_broker(flask_app, url_prefix="/pb", **config):
    '''
    Setup perm broker.
    '''
    setup_conf(**config)
    init_db(**config)
    setup_flask(flask_app, url_prefix=url_prefix)


__ALL__ = ['pb_root', 'conf', 'init_db', 'setup_perm_broker']



# coding: utf-8
import time
import uuid
import flask
from ...common import conf, pb_root
from ..ops import common


# 采用用户名密码作为认证方式
if 'password' in conf.LOGIN_METHODS:

    @pb_root.route('/auth/web/password/login', methods=['GET'])
    def password_login():
        return flask.render_template('password_login.html')

    @pb_root.route('/auth/web/password/reset', methods=['GET'])
    def password_reset():
        return ''


# 采用dingding作为认证方式
if 'dingtalk' in conf.LOGIN_METHODS:
    from ..ops.dingtalk_auth import dingtalk_obj, get_or_create_dingtalk_user


    @pb_root.route('/auth/web/dingtalk/config.js', methods=['GET'])
    def dingtalk_config():
        apis = flask.request.args.get('apis') or ''
        api_list = [i.strip() for i in apis.split(',') if i.strip()]
        url = flask.request.referer
        ret = dingtalk_obj.get_dd_config(url, api_list=api_list)
        api_list = ret.pop('jsApiList')
        js_code = ', '.join("%s: '%s'" % (k, v) for k, v in ret.items())
        js_code += ', jsApiList: %s' % str(api_list)
        js_code = 'dd.config({%s})' % js_code
        return flask.Response(js_code, mimetype='application/javascript')


    @pb_root.route('/auth/web/dingtalk/login', methods=['GET'])
    def dingtalk_login():
        if flask.request.headers.get("X-Forwarded-Proto") == 'https':
            flask.request.environ['wsgi.url_scheme'] = 'https'
        next_url = flask.request.args.get('next')
        if common.is_logined(flask.request, flask.session) and next_url:
            return flask.redirect(next_url)
        url = flask.request.url
        dd_config = dingtalk_obj.get_dd_config(url)
        if 'mobile' in (flask.request.headers.get('User-Agent') or '').lower():
            return flask.render_template('dingtalk_login_mobile.html', dingtalk_data=dd_config)
        return flask.render_template('dingtalk_login.html', dingtalk_data=dd_config)


    @pb_root.route('/auth/web/dingtalk/success', methods=['GET'])
    def dingtalk_success():
        code = flask.request.args.get('code') or ''
        next_url = flask.request.args.get('next') or ''
        status, obj = dingtalk_obj.getuserinfo(code)
        msg = "登陆失败"
        if status and obj and obj.get('userid'):
            userid = obj['userid']
            status, obj = dingtalk_obj.get_userdetail(userid)
            if status:
                user, login = get_or_create_dingtalk_user(userid, obj)
                if user and login and not user.soft_del:
                    common.login(user, flask.request, flask.session)
                    common.record_login(str(user.id), 'dingtalk', **flask.request.headers)
                    msg = "登陆成功"
                    if next_url:
                        homepath = next_url
                    elif 'mobile' in (flask.request.headers.get('User-Agent') or '').lower():
                        homepath = getattr(conf, 'HOMEPATH_MOBILE', None)
                    else:
                        homepath = getattr(conf, 'HOMEPATH', None)
                    if homepath:
                        return flask.redirect(homepath)
                else:
                    msg += ', 未关联钉钉与系统账户'
        return flask.render_template('dingtalk_success.html', msg=msg)



# coding: utf-8
import flask
from ...common import pb_root
from ...util import api_base
from ..ops import common
from ...user.ops import user_base


@pb_root.route('/auth/api/debug/dump', methods=['GET'])
def dump():
    result = {'sid': getattr(flask.session, 'sid', ''),
              'is_logined': common.is_logined(flask.request, flask.session)}
    return api_base.send_json_result("SUCC", result=result)


@pb_root.route('/auth/api/debug/login', methods=['GET'])
def debug_login():
    if common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_LOGIN")
    user = None
    for k, v in flask.request.args.items():
        user = user_base.query_by_info(k, v).first()
        if user:
            break
    if user:
        common.login(user, flask.request, flask.session)
        common.record_login(str(user.id), 'debug', **flask.request.headers)
        return api_base.send_json_result("SUCC")
    else:
        return api_base.send_json_result("AUTH")



import datetime
from mongoengine.queryset.queryset import QuerySet


class DefineQuerySet(QuerySet):

    def update(self, upsert=False, multi=True, write_concern=None,
               full_result=False, **update):
        if hasattr(self._document, "lut"):
            update["lut"] = datetime.datetime.now()
        super(DefineQuerySet, self).update(
            upsert, multi, write_concern, full_result, **update)

    def modify(self, upsert=False, full_response=False, remove=False, new=False, **update):
        if hasattr(self._document, "lut"):
            update["lut"] = datetime.datetime.now()
        super(DefineQuerySet, self).modify(
            upsert, full_response, remove, new, **update)


def change_lut_on_save(document_class):
    '''
    A decorator for Document sub classes.
    If the class have lut field, change it to
    current datetime when save function is called.
    '''
    original_save = document_class.save
    if hasattr(document_class, 'update'):
        original_update = document_class.update
    if hasattr(document_class, 'modify'):
        original_modify = document_class.modify

    def save_with_lut(self, **kw):
        if hasattr(self, 'lut'):
            self.lut = datetime.datetime.now()
        return original_save(self, **kw)

    def update_with_lut(self, **kw):
        if hasattr(self, 'lut'):
            kw["lut"] = datetime.datetime.now()
        original_update(self, **kw)

    def modify_with_lut(self, query=None, **update):
        if hasattr(self, 'lut'):
            update['lut'] = datetime.datetime.now()
        if query is None:
            query = {}
        return original_modify(self, query=query, **update)

    document_class.save = save_with_lut
    document_class._meta["queryset_class"] = DefineQuerySet
    if hasattr(document_class, 'modify'):
        document_class.modify = modify_with_lut
    if hasattr(document_class, 'update'):
        document_class.update = update_with_lut
    return document_class



# -*- coding: utf-8 -*-


class PbException(Exception):
    def __init__(self, *subs):
        # Add a default empty string for subs
        if not subs:
            subs = [""]
        Exception.__init__(self, *subs)



import datetime
import logging
import six
import mongoengine as mg
from . import util
from ... import default
from ...context import flask

logger = logging.getLogger(__name__)


class User(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['keywords']}

    created = mg.DateTimeField(required=True, default=datetime.datetime.now)
    state = mg.StringField(default='normal')
    soft_del = mg.BooleanField(default=False)
    name = mg.StringField()
    info = mg.DictField()
    keywords = mg.ListField()

    def save(self):
        UserAuditLog.record(self, user_id_field_name='id')
        self.keywords = []
        if self.name:
            self.keywords.append(self.name)
        if self.id:
            self.keywords.append(str(self.id))
        for v in self.info.values():
            if v and isinstance(v, six.string_types) and len(v) <= 32 :
                self.keywords.append(v.strip())
        mg.Document.save(self)


class UserAuditLog(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS}

    created = mg.DateTimeField(required=True, default=datetime.datetime.now)
    user_id = mg.StringField(required=True)
    collection_name = mg.StringField()
    editor_id = mg.StringField()
    editor_name = mg.StringField()
    editor_info = mg.DictField()
    fields = mg.ListField()
    delta = mg.ListField()

    @classmethod
    def record(cls, doc, user_id_field_name=None):
        collection_name = doc._collection.name\
                          if doc._collection else str(doc)
        if user_id_field_name:
            user_id = str(getattr(doc, user_id_field_name))
        elif getattr(doc, 'user_id', None):
            user_id = str(doc.user_id)
        else:
            user_id = None
        if not user_id:
            return
        if not getattr(doc, '_changed_fields', None):
            return
        ua_log = UserAuditLog(user_id=user_id,
                              collection_name=collection_name,
                              fields=doc._changed_fields,
                              delta=[])
        for i in doc._delta():
            if not i:
                continue
            for k, v in i.items():
                ua_log.delta.append([k, v])
        try:
            if flask.request and getattr(flask.request, "user", None):
                ua_log.editor_id = str(flask.request.user.id)
                ua_log.editor_name = flask.request.user.name
                ua_log.editor_info = {}
                ua_log.editor_info.update(flask.request.headers)
        except Exception as e:
            logger.exception(e)

        try:
            ua_log.save()
        except Exception as e:
            logger.exception(e)


@util.change_lut_on_save
class PermDesc(mg.Document):
    '''
    权限的描述，用于权限列表的展示，权限的提示等
    '''
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['action', 'name']}
    action = mg.StringField()
    effect = mg.StringField(default="allow")
    resources = mg.ListField()
    name = mg.StringField()
    created = mg.DateTimeField(default=datetime.datetime.now)
    soft_del = mg.BooleanField(default=False)
    lut = mg.DateTimeField(default=datetime.datetime.now)


@util.change_lut_on_save
class RoleDesc(mg.DynamicDocument):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ["foreign_key"]}
    name = mg.StringField()
    perms = mg.ListField()
    granted_positions = mg.ListField()
    foreign_key = mg.StringField()
    info = mg.DictField()
    created = mg.DateTimeField(default=datetime.datetime.now)
    soft_del = mg.BooleanField(default=False)
    lut = mg.DateTimeField(default=datetime.datetime.now)


@util.change_lut_on_save
class PositionDesc(mg.DynamicDocument):
    '''
    职位描述，每个用户通过关联职位描述来确定上下级关系和数据集权限
    --------
    name: string, 职位名称
    parent_id: string, 职位的父级职位id
    path: list, 从根节点到当前节点的id路径，包括根节点，不包括当前节点
    position_type: string, 职位的类型，用于区分一些特殊的职位，比如理财顾问
    '''
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['parent_id',
                        'foreign_key',
                        'tags',
                        'path',
                        'name']}
    name = mg.StringField()
    parent_id = mg.StringField(default='')
    path = mg.ListField()
    position_type = mg.StringField(default='')
    foreign_key = mg.StringField()
    tags = mg.ListField()
    info = mg.DictField()
    created = mg.DateTimeField(default=datetime.datetime.now)
    soft_del = mg.BooleanField(default=False)
    lut = mg.DateTimeField(default=datetime.datetime.now)


@util.change_lut_on_save
class UserPerm(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['user_id',
                        'position_id',
                        'granted_positions']}
    user_id = mg.StringField()
    roles = mg.ListField()
    perms = mg.ListField()
    position_id = mg.StringField()
    granted_positions = mg.ListField()
    created = mg.DateTimeField(default=datetime.datetime.now)
    soft_del = mg.BooleanField(default=False)
    lut = mg.DateTimeField(default=datetime.datetime.now)

    def save(self):
        UserAuditLog.record(self)
        mg.Document.save(self)


class LoginRecord(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['user_id']}

    user_id = mg.StringField(required=True)
    login_type = mg.StringField()
    sid = mg.StringField()
    expiration = mg.DateTimeField()
    info = mg.DictField()
    created = mg.DateTimeField(default=datetime.datetime.now)


@util.change_lut_on_save
class PasswordLogin(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['user_id', 'username']}
    user_id = mg.StringField(required=True)
    username = mg.StringField(required=True)
    password = mg.StringField()
    soft_del = mg.BooleanField(default=False)
    created = mg.DateTimeField(default=datetime.datetime.now)
    lut = mg.DateTimeField(default=datetime.datetime.now)

    def save(self):
        UserAuditLog.record(self)
        Document.save(self)


@util.change_lut_on_save
class ResetPasswordRecord(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['user_id', 'code']}
    user_id = mg.StringField()
    code = mg.StringField()
    query_times = mg.IntField(default=0)
    state = mg.StringField(choices=['created', 'invalid', 'used'],
                           default='created')
    info = mg.DictField()
    timestamp = mg.IntField(default=lambda: int(time.time()))
    created = mg.DateTimeField(default=datetime.datetime.now)
    lut = mg.DateTimeField(default=datetime.datetime.now)


@util.change_lut_on_save
class ThirdPartyLogin(mg.Document):
    meta = {"db_alias": default.MONGOENGINE_ALIAS,
            "indexes": ['user_id', 'ref_id']}
    login_type = mg.StringField()
    user_id = mg.StringField(required=True)
    ref_id = mg.StringField(required=True)
    info = mg.DictField()
    soft_del = mg.BooleanField(default=False)
    created = mg.DateTimeField(default=datetime.datetime.now)
    lut = mg.DateTimeField(default=datetime.datetime.now)



import functools
import mongoengine as mg
from . import models


def parse_key(expr_key):
    return expr_key.replace('.', '__')


def parse_expr(expr_key, expr_value):
    if expr_key == '$or':
        parsed_value = [parse_expr(k, v) for i in expr_value for k, v in i.items()]
        return functools.reduce(lambda x, y: x | y,
                                parsed_value)
    elif expr_key == '$in':
        return mg.Q(**{('%s__in' % parse_key(expr_value[0])): expr_value[1]})
    elif expr_key in ('$ne', '$lt', '$gt'):
        return mg.Q(**{('%s__%s' % (parse_key(expr_value[0])), expr_key): expr_value[1]})
    else:
        return mg.Q(**{parse_key(expr_key): expr_value})


def get_list_query(model_cls, order_by=None, **kwargs):
    q = model_cls.objects
    for k, v in kwargs.items():
        q = q.filter(parse_expr(k, v))
    if order_by:
        q = q.order_by(order_by)
    return q


def get_inst(model_cls, **kwargs):
    inst = get_list_query(model_cls, **kwargs).first()
    return inst



# coding:utf-8
import json
import flask
from ...util import api_base
from ...common import pb_root
from ...user.ops import user_base
from ...auth.ops import common
from ..ops import perm_base
from ..ops import user_perm
from ..ops import position as position_ops
from ..ops.decr import perm_required


@pb_root.route('/perm/api/perm/list', methods=['GET'])
@perm_required()
def api_perm_list():
    '''
    获取当前或者指定用户的全部权限描述。
    '''
    user_id = flask.request.args.get('user_id')
    exclude_role = True if flask.request.args.get('exclude_role') == '1' else False
    if user_id:
        if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:user:get'):
            return api_base.send_json_result("FORBIDDEN")
    else:
        user_id = flask.session["user_id"]
    perm_list = user_perm.get_user_perm_list(user_id, exclude_role=exclude_role)
    return api_base.send_json_result("SUCC",
                               {'perm_list': perm_list,
                                'user_id': user_id})


@pb_root.route('/perm/api/perm/has', methods=['POST'])
@perm_required()
def api_perm_has():
    '''
    检查当前用户或者指定用户是否拥有某一权限。
    '''
    data = flask.request.get_json(force=True)
    perm = data.get('perm')
    target_user_id = data.get('target_user_id')
    target_position_id = data.get('target_position_id')
    user_id = data.get('user_id')

    # Deal with query for other users
    if user_id and not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:user:get'):
        return api_base.send_json_result("FORBIDDEN", msg="需要pauli:perm:user:get权限，请联系管理员")
    if not user_id:
        user_id = flask.session["user_id"]

    if not perm:
        has = False
    else:
        is_upstream = position_ops.is_upstream_position(user_id,
                      position_id=target_position_id, user_id=target_user_id)
        is_owner = position_ops.is_owner(user_id,
                   position_id=target_position_id, user_id=target_user_id)
        has = user_perm.has_perm(user_id, perm,
                                 is_upstream=is_upstream, is_owner=is_owner)
    return api_base.send_json_result("SUCC", {'has': has})


@pb_root.route('/perm/api/perms/has', methods=['POST'])
@perm_required()
def api_perms_has():
    '''
    检查当前用户或者指定用户是否拥有某一个列表的权限
    '''
    data = flask.request.get_json(force=True)
    perms = data.get('perms') or []
    target_user_id = data.get('target_user_id')
    target_position_id = data.get('target_position_id')
    user_id = data.get('user_id')
    return_type = 'list' if data.get('return_type') == 'list' else 'dict'

    # Deal with query for other users
    if user_id and not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:user:get'):
        return api_base.send_json_result("FORBIDDEN", msg="需要pauli:perm:user:get权限，请联系管理员")
    if not user_id:
        user_id = flask.session["user_id"]

    ret = []
    perm_list = user_perm.get_user_perm_list(user_id)
    is_upstream = position_ops.is_upstream_position(user_id,
                  position_id=target_position_id, user_id=target_user_id)
    is_owner = position_ops.is_owner(user_id,
               position_id=target_position_id, user_id=target_user_id)

    for perm in perms:
        has = user_perm.has_perm(user_id, perm,
                                 is_upstream=is_upstream,
                                 is_owner=is_owner,
                                 perm_list=perm_list)
        ret.append((perm, has))
    ret = dict(ret) if return_type == 'dict' else ret
    return api_base.send_json_result("SUCC", {'perms': ret})


@pb_root.route('/perm/api/perm/triple', methods=['GET'])
@perm_required()
def api_perm_triple():
    perm_action = flask.request.args.get('action')
    if not perm_action:
        return api_base.send_json_result("PARAM", msg="必须提供参数action")
    succ, triples = user_perm.get_perm_triples([perm_action], flask.session['user_id'])
    if not succ:
        return api_base.send_json_result("PARAM", msg=triples)
    triple = triples[perm_action]
    if triple.get('+') == True:
        triple['+'] = list(position_ops.get_all_granted_position_ids(flask.session['user_id']))
    if triple.get('-') == True and flask.request.user.info.get('position_id'):
        triple['-'] = flask.session['user_id']
    else:
        triple['-'] = False
    return api_base.send_json_result("SUCC", {'triple': triple})


@pb_root.route('/perm/api/perm/triples', methods=['POST'])
@perm_required()
def api_perm_triples():
    data = flask.request.get_json(force=True)
    perm_actions = data.get('actions')
    if not perm_actions:
        return api_base.send_json_result("PARAM", msg="必须提供参数actions")
    succ, triples = user_perm.get_perm_triples(perm_actions, flask.session['user_id'])
    if not succ:
        return api_base.send_json_result("PARAM", msg=triple)
    position_ids = list(position_ops.get_all_granted_position_ids(flask.session['user_id']))
    for action in triples:
        triple = triples[action]
        if triple.get('+') == True:
            triple['+'] = position_ids
        if triple.get('-') == True and flask.request.user.info.get('position_id'):
            triple['-'] = flask.session['user_id']
        else:
            triple['-'] = False
    return api_base.send_json_result("SUCC", {'triples': triples})


@pb_root.route('/perm/api/perm/user/add', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_perm_grant_to_user():
    data = flask.request.get_json(force=True)
    perm = data.get('perm')
    target_user_id = data.get('user_id')
    if not (perm and target_user_id):
        return api_base.send_json_result("PARAM")
    succ, obj = user_perm.add_perm(perm, user_id=target_user_id)
    if succ:
        return api_base.send_json_result("SUCC", {'perm_list': obj.perms})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/user/update', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_perm_user_upadte():
    data = flask.request.get_json(force=True, silent=True) or {}
    perm = data.get('perms')
    target_user_id = data.get('user_id')
    if not (perms and target_user_id):
        return api_base.send_json_result("PARAM")
    succ, obj = user_perm.update_perms(perms, user_id=target_user_id)
    if succ:
        return api_base.send_json_result("SUCC", {'perm_list': obj.perms})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/user/del', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_remove_perm_from_user():
    data = flask.request.get_json(force=True)
    perm = data.get('perm')
    target_user_id = data.get('user_id')
    succ, obj = user_perm.remove_perm(perm, user_id=target_user_id)
    if succ:
        return api_base.send_json_result("SUCC", {'perm_list': obj.perms})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/user/position', methods=['GET'])
def api_get_user_position():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    user_id = flask.request.args.get("user_id")
    if user_id and not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:user:get'):
        return api_base.send_json_result("FORBIDDEN", msg="用户没有权限pauli:perm:user:get")
    user_id = user_id or flask.session['user_id']
    positions = list(position_ops.get_granted_position_ids(user_id,
                                                           include_roles=False))
    return api_base.send_json_result("SUCC", result={'positions': positions})


@pb_root.route('/perm/api/perm/user/position', methods=['POST'])
@perm_required("pauli:perm:user:update")
def api_set_user_position():
    data = flask.request.get_json(force=True)
    user_id = data.get('user_id')
    position_id = data.get('position_id')
    if not user_id:
        return api_base.send_json_result("PARAM", msg="user_id未提供")
    status, obj = position_ops.set_position(user_id, position_id)
    if status:
        return api_base.send_json_result("SUCC", result={'position_id': obj.position_id})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/user/position/grant', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_grant_user_position():
    data = flask.request.get_json(force=True)
    user_id = data.get('user_id')
    position_id = data.get('position_id')
    if not (user_id and position_id):
        return api_base.send_json_result("PARAM",
                msg="user_id或者position_id未提供")
    status, obj = position_ops.grant_position('add', position_id, user_id=user_id)
    if status:
        return api_base.send_json_result("SUCC", result={'grant_positions': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/user/position/recall', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_recall_user_position():
    data = flask.request.get_json(force=True)
    user_id = data.get('user_id')
    position_id = data.get('position_id')
    if not (user_id and position_id):
        return api_base.send_json_result("PARAM",
                msg="user_id或者position_id未提供")
    status, obj = position_ops.grant_position('del', position_id, user_id=user_id)
    if status:
        return api_base.send_json_result("SUCC", result={'grant_positions': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/user/position/update', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_update_user_positions():
    data = flask.request.get_json(force=True, silent=True) or {}
    user_id = data.get('user_id')
    position_ids = data.get('position_ids')
    if not (user_id and position_ids):
        return api_base.send_json_result("PARAM",
                                   msg="user_id或者position_ids未提供")
    if not type(position_ids) is list:
        return api_base.send_json_result("PARAM",
                                   msg="position_ids必须为list")
    status, obj = position_ops.set_granted_positions(user_id, position_ids)
    if status:
        return api_base.send_json_result("SUCC",
                                   result={'grant_positions': obj.granted_positions})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/role/position/grant', methods=['PUT'])
@perm_required("pauli:perm:role:update")
def api_grant_role_position():
    data = flask.request.get_json(force=True)
    role_id = data.get('role_id')
    position_id = data.get('position_id')
    if not (role_id and position_id):
        return api_base.send_json_result("PARAM",
                msg="role_id或者position_id未提供")
    status, obj = position_ops.grant_position('add', position_id, role_id=role_id)
    if status:
        return api_base.send_json_result("SUCC", result={'grant_positions': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/role/position/recall', methods=['PUT'])
@perm_required("pauli:perm:role:update")
def api_recall_role_position():
    data = flask.request.get_json(force=True)
    role_id = data.get('role_id')
    position_id = data.get('position_id')
    if not (role_id and position_id):
        return api_base.send_json_result("PARAM",
                msg="role_id或者position_id未提供")
    status, obj = position_ops.grant_position('del', position_id, role_id=role_id)
    if status:
        return api_base.send_json_result("SUCC", result={'grant_positions': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/role/add', methods=['PUT'])
def api_grant_perm_to_role():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role:update'):
        return api_base.send_json_result("FORBIDDEN")
    data = flask.request.get_json(force=True)
    perm = data.get('perm')
    target_role_id = data.get('role_id')
    if not (perm and target_role_id):
        return api_base.send_json_result("PARAM")
    succ, obj = user_perm.add_perm(perm, role_id=target_role_id)
    if succ:
        return api_base.send_json_result("SUCC", result={'perm_list': obj.perms})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/perm/role/del', methods=['PUT'])
def api_remove_perm_from_role():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role:update'):
        return api_base.send_json_result("FORBIDDEN")
    data = flask.request.get_json(force=True)
    perm = data.get('perm')
    target_role_id = data.get('role_id')
    succ, obj = user_perm.remove_perm(perm, role_id=target_role_id)
    if succ:
        return api_base.send_json_result("SUCC", result={'perm_list': obj.perms})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/role/list', methods=['GET'])
def api_role_list():
    '''
    获取全部角色列表。
    '''
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role'):
        return api_base.send_json_result("FORBIDDEN")
    roles = user_perm.get_all_roles()
    roles_list = map(lambda x: {'id': str(x.id), 'name': x.name}, roles)
    return api_base.send_json_result("SUCC", result={'roles': list(roles_list)})


@pb_root.route('/perm/api/role', methods=['GET'])
def api_get_role():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role:get'):
        return api_base.send_json_result("FORBIDDEN")
    role_id = flask.request.args.get('role_id')
    succ, obj = user_perm.get_role_info(role_id=role_id)
    if succ:
        return api_base.send_json_result("SUCC", result={'role': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/role', methods=['POST'])
def api_create_role():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role:create'):
        return api_base.send_json_result("FORBIDDEN")
    data = flask.request.get_json(force=True)
    name = data.get('name')
    succ, obj = user_perm.create_role(name=name)
    if succ:
        return api_base.send_json_result("SUCC", result={'role': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/role', methods=['DELETE'])
def api_remove_role():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role:del'):
        return api_base.send_json_result("FORBIDDEN")
    role_id = flask.request.args.get('role_id')
    succ, obj = user_perm.remove_role(role_id)
    if succ:
        return api_base.send_json_result("SUCC", result={'role': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/role', methods=['PUT'])
def api_update_role():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:role:update'):
        return api_base.send_json_result("FORBIDDEN")
    data = flask.request.get_json(force=True)
    role_id = data.get('role_id')
    name = data.get('name')
    succ, obj = user_perm.update_role(role_id, name=name)
    if succ:
        return api_base.send_json_result("SUCC", result={'role': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/role/user/list', methods=['GET'])
def api_get_role_list():
    '''
    获取当前或者指定用户的角色id列表。
    '''
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    user_id = flask.request.args.get('user_id')
    if user_id and not user_perm.has_perm(flask.session["user_id"],
                                          'pauli:perm:user:get'):
        return api_base.send_json_result("FORBIDDEN", msg="用户没有权限pauli:perm:user:get")
    if not user_id:
        user_id = flask.session['user_id']
    roles = user_perm.get_user_roles(user_id)
    roles_list = map(lambda x: {'id': str(x.id), 'name': x.name}, roles)
    return api_base.send_json_result("SUCC", result={'roles': list(roles_list)})


@pb_root.route('/perm/api/role/user/update', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_update_user_role():
    data = flask.request.get_json(force=True, silent=True) or {}
    user_id = data.get('user_id')
    role_ids = data.get('role_ids') or []
    if not (user_id and role_ids):
        return api_base.send_json_result("FORBIDDEN")
    succ, obj = user_perm.update_user_roles(user_id, role_ids)
    if succ:
        roles = user_perm.get_user_roles(user_id)
        roles_list = list(map(lambda x: {'id': str(x.id), 'name': x.name}, roles))
        return api_base.send_json_result("SUCC",
                                   result={'roles': roles_list})
    else:
        return api_base.send_json_result("PARAM",
                                   msg=obj)


@pb_root.route('/perm/api/role/user', methods=['PUT'])
@perm_required("pauli:perm:user:update")
def api_grant_role_to_user():
    data = flask.request.get_json(force=True)
    role_id = data.get('role_id')
    user_id = data.get('user_id')
    succ, obj = user_perm.add_role_to_user(role_id, user_id)
    if succ:
        roles = user_perm.get_user_roles(user_id)
        roles_list = list(map(lambda x: {'id': str(x.id), 'name': x.name}, roles))
        return api_base.send_json_result("SUCC", result={'roles': roles_list})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/role/user', methods=['DELETE'])
def api_remove_role_from_user():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:user:update'):
        return api_base.send_json_result("FORBIDDEN", msg="用户没有权限pauli:perm:user:update")
    role_id = flask.request.args.get('role_id')
    user_id = flask.request.args.get('user_id')
    succ, obj = user_perm.remove_role_from_user(role_id, user_id)
    if succ:
        roles = user_perm.get_user_roles(user_id)
        roles_list = list(map(lambda x: {'id': str(x.id), 'name': x.name}, roles))
        return api_base.send_json_result("SUCC", result={'roles': roles_list})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/position/users', methods=['GET', 'POST'])
@perm_required()
def api_position_users():
    position_ids = []
    if flask.request.method == "GET":
        position_id = flask.request.args.get('position_id')
        if position_id:
            position_ids.append(position_id)
    elif flask.request.method == "POST":
        data = flask.request.get_json(force=True)
        position_ids = data.get('position_ids') or []
    if not position_ids:
        granted_position_ids = list(position_ops.get_granted_position_ids(flask.session["user_id"]))
    else:
        if not position_ops.is_upstream_position(flask.request.user,
                                                 position_ids=position_ids)\
                and not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:position:list'):
            return api_base.send_json_result("FORBIDDEN", msg="用户不是所传职位的上级用户")
        granted_position_ids = position_ids
    users = position_ops.get_users_by_sub_positions(granted_position_ids, included=True)
    ret = [user_base.to_dict(user) for user in users]
    return api_base.send_json_result("SUCC", result={'users': ret})


@pb_root.route('/perm/api/position/list', methods=['GET', 'POST'])
@perm_required()
def api_position_list():
    position_ids = []
    if flask.request.method == "GET":
        position_id = flask.request.args.get('position_id')
        if position_id:
            position_ids.append(position_id)
    elif flask.request.method == "POST":
        data = flask.request.get_json(force=True)
        position_ids = data.get('position_ids') or []
    if not position_ids:
        all_position_ids = list(position_ops.get_all_granted_position_ids(flask.session["user_id"]))
    else:
        if not position_ops.is_upstream_position(flask.request.user,
                                                 position_ids=position_ids)\
                and not user_perm.has_perm(flask.session["user_id"], 'pauli:perm:position:list'):
            return api_base.send_json_result("FORBIDDEN", msg="用户不是所传职位的上级用户")
        all_positions = position_ops.get_sub_positions(position_ids, included=True)
        all_position_ids = list(map(lambda x: str(x.id), all_positions))
    return api_base.send_json_result("SUCC", result={'position_list': all_position_ids})


@pb_root.route('/perm/api/position/leaders/info', methods=['GET'])
@perm_required()
def api_position_leaders_info():
    '''
    获取一个团队的leader的info.
    '''
    position_id = flask.request.args.get('position_id') or ''
    if not position_id:
        return api_base.send_json_result("PARAM",
                                   msg="必须传递position_id")
    user_ids = position_ops.get_position_leaders(position_id)
    succ, obj = user_base.get_user_info_dict(user_ids)
    if succ:
        return api_base.send_json_result("SUCC", {'info': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/position/tree', methods=['GET'])
@perm_required()
def api_position_tree():
    extend = True if flask.request.args.get('extend') == '1' else False
    position_id = flask.request.args.get('position_id')
    extra_fields = flask.request.args.get('extra_fields') or []
    if extra_fields:
        extra_fields = extra_fields.split(',')
    granted_position_ids = position_ops.get_granted_position_ids(flask.session["user_id"])
    if position_id:
        if position_id in granted_position_ids:
            target_position_ids = [position_id]
        elif user_perm.has_perm(flask.session["user_id"], 'pauli:perm:position:list'):
            target_position_ids = [position_id]
        else:
            return api_base.send_json_result("FORBIDDEN", msg="用户不是所传职位的上级用户")
    else:
        target_position_ids = granted_position_ids
    ret = position_ops.position_tree(target_position_ids,
                                     extend_users=extend,
                                     extra_fields=extra_fields)
    return api_base.send_json_result("SUCC", result={'tree': ret})


@pb_root.route('/perm/api/position/dump', methods=['GET'])
@perm_required(perms="pauli:perm:position:dump")
def api_position_dump():
    ret = position_ops.position_tree_dump()
    return api_base.send_json_result("SUCC", result={'dump': ret})


@pb_root.route('/perm/api/position', methods=['GET'])
@perm_required()
def api_position_get():
    position_id = flask.request.args.get("position_id")
    if not position_id:
        return api_base.send_json_result("PARAM")
    position_info = position_ops.get_position_info(position_id)
    return api_base.send_json_result("SUCC",
               result={'position': position_info})


@pb_root.route('/perm/api/positions', methods=['PUT'])
@perm_required()
def api_positions_get():
    data = flask.request.get_json(force=True)
    position_ids = data.get('position_ids')
    if not position_ids:
        return api_base.send_json_result("PARAM")
    position_info_dict = {}
    for position_id in position_ids:
        position = position_ops.get_position_info(position_id)
        if position:
            position_info_dict[position_id] = position
    return api_base.send_json_result("SUCC",
               result={'positions': position_info_dict})


@pb_root.route('/perm/api/position', methods=['POST'])
@perm_required(perms="pauli:perm:position:add")
def api_position_add():
    data = flask.request.get_json(force=True)
    name = data.get('name')
    parent_id = data.get('parent_id') or None
    parent_name = data.get('parent_name') or None
    if not name:
        return api_base.send_json_result("PARAM", msg="未提供职位或者团队的name")
    status, obj = position_ops.add_position(name,
                                            parent_id=parent_id,
                                            parent_name=parent_name)
    if status:
        return api_base.send_json_result("SUCC",
                result={'position': position_ops.position_to_dict(obj)})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/position', methods=['DELETE'])
@perm_required(perms="pauli:perm:position:delete")
def api_position_delete():
    position_id = flask.request.args.get('position_id')
    if not position_id:
        return api_base.send_json_result("PARAM", msg="未提供职位或者团队id")
    status, obj = position_ops.remove_position(position_id)
    if status:
        return api_base.send_json_result("SUCC",
                result={'position': position_ops.position_to_dict(obj)})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/perm/api/position', methods=['PUT'])
@perm_required(perms="pauli:perm:position:update")
def api_position_update():
    data = flask.request.get_json(force=True)
    position_id = data.get('position_id')
    new_name = data.get('new_name') or ''
    parent_id = data.get('parent_id')

    if position_id and new_name:
        status, obj = position_ops.rename_position(position_id,
                      new_name)
    elif position_id and parent_id is not None:
        status, obj = position_ops.move_position(position_id,
                      target_parent_id=parent_id)
    else:
        return api_base.send_json_result("PARAM", msg="未提供职位或者团队id")
    if status:
        return api_base.send_json_result("SUCC",
                result={'position': position_ops.position_to_dict(obj)})
    else:
        return api_base.send_json_result("PARAM", msg=obj)



# coding:utf-8
import hashlib
import logging
import requests
from ... import conf
from ... util.cache_util import cache_lock, cache
from ...user.models import User
from ..models import DingtalkLogin
from dingle.interface import get_api_client
import dingle.dingtalk.user as dingtalk_user_api

logger = logging.getLogger('dingtalk')
dingtalk_obj = None


class DingtalkApi(object):
    def __init__(self, **kwargs):
        token_key_prefix = kwargs.get('corp_id') or ''
        self.api_client = get_api_client(config_dict=kwargs,
                                         token_store=cache,
                                         token_key_prefix=token_key_prefix)

    def _api(self, method, path, **kw):
        url = '%s%s' % ('https://oapi.dingtalk.com', path)
        if method == 'GET':
            resp = requests.get(url, **kw)
        elif method == 'POST':
            resp = requests.post(url, **kw)
        else:
            resp = None
        logger.info({"action": "dingtalkApiCall",
                     "method": method, "path": path, "kw": kw,
                     "content": resp.content})
        return resp

    def get_access_token(self):
        return self.api_client.manager.get_access_token()

    def get_jsapi_ticket(self):
        return self.api_client.manager.get_jsapi_ticket()

    def getuserinfo(self, code):
        ret = dingtalk_user_api.getuserinfo(code)
        if ret['errcode'] == 0:
            return True, ret
        return False, ret

    def get_userdetail(self, userid):
        ret = dingtalk_user_api.get(userid)
        if ret['errcode'] == 0:
            return True, ret
        return False, ret

    def sign_js_api(self, noncestr, timestamp, jsapi_ticket, url):
        pairs = [('jsapi_ticket', jsapi_ticket), ('noncestr', noncestr),
                 ('timestamp', timestamp), ('url', url)]
        sign_str = '&'.join(["%s=%s" % (k, v) for (k, v) in pairs]).encode('utf-8')
        signature = hashlib.sha1(sign_str).hexdigest()
        return signature

    def get_dd_config(self, url, api_list=None):
        return self.api_client.manager.get_dd_config(url,
                                                     api_list=api_list)


if getattr(conf, 'DINGTALK', None):
    dingtalk_obj = DingtalkApi(**conf.DINGTALK)


def set_user_info_from_dingtalk(user, dingtalk_info, overwrite=None):
    '''
    Set user.info based on dingtalk_info.
    '''
    overwrite = overwrite if overwrite is not None\
                else conf.USER_INFO_DINGTALK_DEFAULT_OVERWRITE
    fields = conf.USER_INFO_DINGTALK_FIELDS
    update_user_info = {}
    update_user_info['dingtalk_id'] = dingtalk_info['userid']
    for field in fields:
        if user.info.get(field) != dingtalk_info.get(field):
            if user.info.get(field) and not overwrite:
                continue
            if field == 'name':
                user.name = dingtalk_info.get(field)
            update_user_info[field] = dingtalk_info.get(field)
    user.info = {**user.info, **update_user_info}


def get_user_by_dingtalk_info(dingtalk_info):
    '''
    Get a user base on conf.USER_DINGTALK_PRIMARY_FIELDS.
    '''
    fields = getattr(conf, "USER_DINGTALK_PRIMARY_FIELDS", [])
    for field in fields:
        value = dingtalk_info.get(field)
        if not value:
            continue
        user = User.objects(**{("info__%s" % field): value,
                               "soft_del": False}).first()
        if user:
            return user
    return None


def get_or_create_dingtalk_user(dingtalk_id, dingtalk_info):
    '''
    Get or create an dingtalk login object,
    if conf.DINGTALK_AUTO_CREATE_USER is enabled, a new pauli
    user will be created when no user is matched current
    dingtalk login object.
    The fucntion shall return a (user, login) 2-tuple.
    '''
    user = None
    with cache_lock('dingtalk_%s' % dingtalk_id):
        login = DingtalkLogin.objects(dingtalk_id=dingtalk_id).first()
        if login:
            user = User.objects(id=login.user_id,
                                soft_del=False).first()
            if user:
                set_user_info_from_dingtalk(user, dingtalk_info)
                user.save()
                login.info = dingtalk_info
                login.save()
        else:
            user = get_user_by_dingtalk_info(dingtalk_info)
            if user:
                set_user_info_from_dingtalk(user, dingtalk_info)
                user.save()
                login = DingtalkLogin(user_id=str(user.id),
                                      dingtalk_id=dingtalk_id,
                                      info=dingtalk_info)
                login.save()
            elif getattr(conf, 'DINGTALK_AUTO_CREATE_USER', None):
                user = User(name=dingtalk_info.get('name'))
                set_user_info_from_dingtalk(user, dingtalk_info)
                user.save()
                login = DingtalkLogin(user_id=str(user.id),
                                      dingtalk_id=dingtalk_id,
                                      info=dingtalk_info)
                login.save()
            if login:
                logger.info({"action": "createDingtalkUser",
                             "user_id": str(user.id),
                             "login_id": str(login.id),
                             "info": dingtalk_info})
        return user, login



class DbBridge(object):
    models = None
    helper = None

    def __getattr__(self, name):
        if hasattr(self.models, name):
            return getattr(self.models, name)
        else:
            return getattr(self.helper, name)

db = DbBridge()


def init_db(**config):
    backend_type = config.get('BACKEND_TYPE')
    if backend_type == 'mongodb':
        import mongoengine
        mongoengine.register_connection(config['MONGO_ALIAS'],
                                        **config['MONGO'])
        from .mongodb import models
        from .mongodb import helper
        db.models = models
        db.helper = helper
        return db
    elif backend_type == 'mssql':
        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker
        engine = create_engine(config['CONNECTION_STRING'], **config['SA_ENGINE_ARGS'])
        session = scoped_session(sessionmaker(bind=engine))
        from .mssql import models
        from .mssql import helper
        models.engine = helper.engine = engine
        models.session = helper.session = session
        db.models = models
        db.helper = helper
        return db
    elif backend_type == 'mysql':
        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker
        engine = create_engine(config['CONNECTION_STRING'], **config['SA_ENGINE_ARGS'])
        session = scoped_session(sessionmaker(bind=engine))
        from .mysql import models
        from .mysql import helper
        models.engine = helper.engine = engine
        models.session = helper.session = session
        db.models = models
        db.helper = helper
        return db
    else:
        pass



import functools
from sqlalchemy.schema import CreateTable
from .models import *


def parse_key(expr_key, model_cls):
    if '.' in expr_key:
        full_path = expr_key.split('.')
        field = full_path[0]
        path = '.'.join(['$', *full_path[1:]])
        return sa.func.json_value(getattr(model_cls, field), path)
    else:
        return getattr(model_cls, expr_key)


def parse_expr(expr_key, expr_value, model_cls):
    if expr_key == '$or':
        return sa.or_(*[parse_expr(k, v, model_cls)\
                        for i in expr_value for k, v in i.items()])
    elif expr_key == '$in':
        return parse_key(expr_value[0], model_cls).in_(expr_value[1])
    elif expr_key == '$ne':
        return parse_key(expr_value[0], model_cls) != (expr_value[1])
    elif expr_key == '$lt':
        return parse_key(expr_value[0], model_cls) < (expr_value[1])
    elif expr_key == '$gt':
        return parse_key(expr_value[0], model_cls) > (expr_value[1])
    else:
        return parse_key(expr_key, model_cls) == expr_value


def create_table():
    ret = []
    for k, v in globals().items():
        if hasattr(v, '__table__'):
            ret.append(str(CreateTable(v.__table__).compile(engine)).strip() + ';')
    return '\n\n'.join(ret)


def get_list_query(model_cls, order_by=None, **kwargs):
    q = session.query(model_cls)
    for k, v in kwargs.items():
        q = q.filter(parse_expr(k, v, model_cls))
    if order_by:
        if isinstance(order_by, str) and order_by.startswith('-'):
            q = q.order_by(parse_key(order_by[1:], model_cls))
        elif isinstance(order_by, str):
            q = q.order_by(parse_key(order_by, model_cls))
    return q


def get_inst(model_cls, **kwargs):
    inst = get_list_query(model_cls, **kwargs).first()
    return inst



# coding: utf-8
import time
import json
import uuid
import flask
from ...common import conf, pb_root
from ...util import api_base
from ..ops import common
from ..ops import  pass_auth
from ...user.ops import user_base
from ...perm.ops import user_perm
from ...perm.ops.decr import perm_required


@pb_root.route('/auth/api/logined', methods=['GET'])
def logined():
    logined = common.is_logined(flask.request, flask.session)
    result = {'logined': logined}
    if logined:
        result.update(user_base.to_dict(flask.request.user))
    return api_base.send_json_result("SUCC", result=result)


@pb_root.route('/auth/api/logout', methods=['GET'])
def logout():
    common.logout(flask.request, flask.session)
    resp = api_base.send_json_result("SUCC")
    resp.set_cookie(conf.SESSION_COOKIE_NAME, "", expires=0)
    return resp


@pb_root.route('/auth/api/record', methods=['GET'])
@perm_required("pauli:user:audit")
def api_login_record():
    '''
    获取用户的登录记录
    '''
    user_id = flask.request.args.get('user_id')
    cursor_id = flask.request.args.get('cursor_id') or ''
    limit = int(flask.request.args.get('limit') or 10)
    if not user_id:
        return api_base.send_json_result("PARAM")
    status, obj = common.get_user_login_record(user_id,
                                               cursor_id=cursor_id,
                                               limit=limit)
    if status:
        return api_base.send_json_result("SUCC", result={'logs': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/auth/api/pass/login', methods=['PUT'])
def pass_login():
    if common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_LOGIN")
    data = flask.request.get_json(force=True)
    username = data.get('username')
    password = data.get('password')
    user = pass_auth.get_user(username, password)
    if user:
        flask.session['password_verified'] = {
            'timestamp': int(time.time()),
            'user_id': str(user.id),
            'mobile': user.info.get('mobile', ''),
        }
        if pass_auth.should_start_phone_verify(user, flask.request):
            common.record_login(str(user.id), 'password-sms', **flask.request.headers)
            return api_base.send_json_result("USER_NEED_SMS_AUTH", {})
        else:
            common.login(user, flask.request, flask.session)
            common.record_login(str(user.id), 'password-succ', **flask.request.headers)
            return api_base.send_json_result("SUCC")
    else:
        if 'password_verified' in flask.session:
            del flask.session['password_verified']
        return api_base.send_json_result("USER_NAME_OR_PWD_ERROR")


@pb_root.route('/auth/api/pass/code', methods=['GET'])
def pass_send_code():
    if common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_LOGIN")
    method = flask.request.args.get('method') or 'voice'
    succ, reason = pass_auth.send_sms_code(flask.session, method=method)
    if succ:
        return api_base.send_json_result("SUCC", {})
    else:
        return api_base.send_json_result("PARAM", msg=reason)


@pb_root.route('/auth/api/pass/code', methods=['POST'])
def pass_verify_code():
    if common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_LOGIN")
    data = flask.request.get_json(force=True)
    code = data.get('code')
    succ, user = pass_auth.verify_sms_code(flask.session, code)
    if succ:
        common.login(user, flask.request, flask.session)
        common.record_login(str(user.id), 'sms-succ', **flask.request.headers)
        return api_base.send_json_result("SUCC", {})
    else:
        return api_base.send_json_result("USER_VERIFY_CODE_ERROR")


@pb_root.route('/auth/api/pass/change', methods=['POST'])
def change():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    data = flask.request.get_json(force=True)
    password = data.get('password')
    new_password = data.get('new_password')
    if not (password and new_password):
        return api_base.send_json_result("PARAM")
    username = pass_auth.get_username(flask.request.user)
    if not username:
        return api_base.send_json_result("USER_NAME_OR_PWD_ERROR")
    user = pass_auth.get_user(username, password)
    if user and user.id == flask.request.user.id:
        status, obj = pass_auth.change_password(user, new_password)
        if not status:
            return api_base.send_json_result("PARAM", msg=obj)
        else:
            return api_base.send_json_result("SUCC")
    else:
        return api_base.send_json_result("USER_NAME_OR_PWD_ERROR")


@pb_root.route('/auth/api/pass/create', methods=['POST'])
def create():
    if not common.is_logined(flask.request, flask.session):
        return api_base.send_json_result("USER_NOT_LOGIN")
    if not user_perm.has_perm(flask.session["user_id"], 'pauli:user:create'):
        return api_base.send_json_result("FORBIDDEN")
    data = flask.request.get_json(force=True)
    username = (data.get('username') or '').lower()
    password = data.get('password') or str(uuid.uuid1().hex)
    name = data.get('name')
    mobile = data.get('mobile') or ''
    creator_id = flask.session["user_id"]
    creator_name = flask.request.user.name
    if not (username and password and name):
        return api_base.send_json_result("PARAM", msg="提供的参数不足")
    status, obj, msg = pass_auth.create_user_by_pass(
        username, password, name=name, mobile=mobile,
        creator_id=creator_id, creator_name=creator_name)
    if status:
        return api_base.send_json_result("SUCC", result={'user_id': str(obj.id)})
    else:
        return api_base.send_json_result("PARAM", msg=msg)


@pb_root.route('/auth/api/pass/reset', methods=['GET'])
def pass_reset_get():
    email = flask.request.args.get('email')
    if not email:
        return api_base.send_json_result("PARAM")
    email_addr_base = pass_auth.get_reset_addr_base(flask.request)
    succ, msg = pass_auth.send_reset_email(email, email_addr_base)
    if succ:
        return api_base.send_json_result("SUCC")
    else:
        return api_base.send_json_result("PARAM", msg=msg)


@pb_root.route('/auth/api/pass/reset', methods=['PUT'])
def pass_reset_verify():
    data = flask.request.get_json(force=True)
    code = data.get('code')
    if not code:
        return api_base.send_json_result("PARAM")
    succ, msg = pass_auth.verify_reset_code(code)
    if succ:
        return api_base.send_json_result("SUCC")
    else:
        return api_base.send_json_result("PARAM", msg=msg)


@pb_root.route('/auth/api/pass/reset', methods=['POST'])
def pass_reset():
    data = flask.request.get_json(force=True)
    code = data.get('code')
    password = data.get('password') or ''
    mobile = data.get('mobile') or ''
    if not code:
        return api_base.send_json_result("PARAM")
    succ, msg = pass_auth.reset_record(code, password=password, mobile=mobile)
    if succ:
        return api_base.send_json_result("SUCC")
    else:
        return api_base.send_json_result("PARAM", msg=msg)


if 'dingtalk' in getattr(conf, 'LOGIN_METHODS', []):
    from ..ops.dingtalk_auth import dingtalk_obj

    @pb_root.route('/auth/api/dingtalk/jsapi_ticket')
    def dingtalk_jsapi_ticket():
        jsapi_ticket = dingtalk_obj.get_jsapi_ticket()
        return api_base.send_json_result("SUCC", result={'jsapi_ticket': jsapi_ticket})



# coding:utf-8
import json
import flask
from ...util import api_base
from ...common import pb_root
from ..ops import user_base
from ...auth.ops import common
from ...perm.ops import user_perm
from ...perm.ops.decr import perm_required


@pb_root.route('/user/api/list', methods=['GET'])
@perm_required()
def api_user_list():
    '''
    获取全部用户列表，不包括已经删除的用户
    '''
    keyword = flask.request.args.get('keyword') or ''
    enabled = flask.request.args.get('enabled') or ''
    if enabled in ['0', '1']:
        query = {'soft_del': {'1': False, '0': True}[enabled]}
    else:
        query = {'soft_del__in': [False, True]}
    keywords = [i.strip() for i in keyword.split() if i.strip()]
    if keywords:
        query['keywords__in'] = keywords
    succ, obj = user_base.list_users(q=query)
    if succ:
        return api_base.send_json_result("SUCC", {'user_list': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/info/update', methods=['PUT'])
@perm_required("pauli:user:update")
def api_user_info_update():
    '''
    更新用户信息
    '''
    data = flask.request.get_json(force=True, silent=True) or {}
    user_id = data.get('user_id')
    name = data.get('name')
    if not (user_id and name):
        return api_base.send_json_result("PARAM", msg='必须提供user_id')
    succ, obj = user_base.update_user(user_id, name=name)
    if succ:
        return api_base.send_json_result("SUCC", {'user': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/info', methods=['POST'])
@perm_required()
def api_user_info():
    '''
    根据id获取用户的名字
    '''
    data = flask.request.get_json(force=True)
    user_ids = data.get("user_ids")
    if not user_ids:
        return api_base.send_json_result("PARAM", msg="必须传递用户id列表")
    succ, obj = user_base.get_user_info_dict(user_ids)
    if succ:
        return api_base.send_json_result("SUCC", {'info': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/names', methods=['POST'])
@perm_required()
def api_user_names():
    '''
    根据id获取用户的名字
    '''
    data = flask.request.get_json(force=True)
    user_ids = data.get("user_ids")
    if not user_ids:
        return api_base.send_json_result("PARAM", msg="必须传递用户id列表")
    succ, obj = user_base.get_user_names(user_ids)
    if succ:
        return api_base.send_json_result("SUCC", {'user_names': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/query', methods=['GET'])
@perm_required("pauli:user:list")
def api_user_query():
    '''
    根据用户名或者用户姓名等信息筛选相关用户，包括已经disabled掉的用户
    '''
    keyword = flask.request.args.get('keyword')
    if not keyword:
        return api_base.send_json_result("PARAM")
    succ, obj = user_base.query_users(keyword)
    if succ:
        return api_base.send_json_result("SUCC", {'user_list': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/query/byinfo', methods=['GET'])
@perm_required()
def api_user_query_by_info():
    '''
    根据用户info中的信息查询用户列表，仅限email、mobile、jobnumber、
    position_name、position_tag和role_name
    '''
    query_type = flask.request.args.get('query_type')
    query_value = flask.request.args.get('query_value')
    permitted_types = ['email',
                       'mobile',
                       'jobnumber',
                       'position_name',
                       'position_tag',
                       'role_name']
    if query_type not in permitted_types or not query_value:
        return api_base.send_json_result("PARAM")
    query = user_base.query_by_info(query_type, query_value)
    users = [user_base.to_dict(user) for user in query]
    return api_base.send_json_result("SUCC", {'users': users})


@pb_root.route('/user/api/get/byinfo', methods=['GET'])
@perm_required()
def api_user_get_by_info():
    '''
    根据用户info中的信息查询单个用户，仅限email和mobile
    '''
    query_type = flask.request.args.get('query_type')
    query_value = flask.request.args.get('query_value')
    if query_type not in ['email', 'mobile'] or not query_value:
        return api_base.send_json_result("PARAM")
    succ, obj = user_base.get_by_info(query_type, query_value)
    if succ:
        return api_base.send_json_result("SUCC", {'user': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/enabled', methods=['GET'])
@perm_required("pauli:user:update")
def api_user_enable():
    '''
    使停用的用户恢复成enabled状态.
    '''
    user_id = flask.request.args.get('user_id')
    if not user_id:
        return api_base.send_json_result("PARAM")
    status, obj = user_base.enable_user(user_id)
    if status:
        return api_base.send_json_result("SUCC", result={'user': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/enabled', methods=['DELETE'])
@perm_required("pauli:user:update")
def api_user_disable():
    '''
    停用现有用户.
    '''
    user_id = flask.request.args.get('user_id')
    if not user_id:
        return api_base.send_json_result("PARAM")
    status, obj = user_base.disable_user(user_id)
    if status:
        return api_base.send_json_result("SUCC", result={'user': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)


@pb_root.route('/user/api/audit', methods=['GET'])
@perm_required("pauli:user:audit")
def api_user_audit():
    '''
    获取用户的audit log
    '''
    user_id = flask.request.args.get('user_id')
    cursor_id = flask.request.args.get('cursor_id') or ''
    limit = int(flask.request.args.get('limit') or 10)
    if not user_id:
        return api_base.send_json_result("PARAM")
    status, obj = user_base.get_user_audit_log(user_id,
                                               cursor_id=cursor_id,
                                               limit=limit)
    if status:
        return api_base.send_json_result("SUCC", result={'logs': obj})
    else:
        return api_base.send_json_result("PARAM", msg=obj)



import re
import six

ACTION_PATTERN = re.compile(r"^[_a-zA-Z0-9-]+$")


def is_valid_action(action_str):
    '''
    Check whether an action_str is valid.
    Valid action examples:
        "gesafe:storage:*"
        "tianfu:article:write"
    '''
    segment_list = action_str.strip().split(":")
    if not segment_list:
        return False
    for segment in segment_list[:-1]:
        if not (ACTION_PATTERN.match(segment) or segment == '*'):
            return False
    if (not ACTION_PATTERN.match(segment_list[-1])) and segment_list[-1] != '*':
        return False
    return True


def is_valid_resource(resource_str):
    '''
    Check whether an resource_str is a valid resource description.
    Valid resource examples
        "gesafe:article:*"
        "gesafe:article:+"
        "gesafe:article:-"
        "tianfu:article:write"
    '''
    segment_list = resource_str.strip().split(":")
    if not segment_list:
        return False
    for segment in segment_list[:-1]:
        if not ACTION_PATTERN.match(segment):
            return False
    if (not ACTION_PATTERN.match(segment_list[-1]))\
            and segment_list[-1] not in ['*', '+', '-']:
        return False
    return True


def is_action_matched(user_action, target_action):
    '''
    Check whether an action provided as user_action can match
    a target_action.
    '''
    if not is_valid_action(user_action):
        return False
    user_list = user_action.strip().split(":")
    target_list = target_action.strip().split(":")
    for idx, segment in enumerate(user_list):
        if idx >= len(target_list):
            return False
        elif user_list[idx] != "*" and user_list[idx] != target_list[idx]:
            return False
    if len(user_list) < len(target_list) and\
             user_list and user_list[-1] != '*':
        return False
    return True


def is_resource_matched(user_resource, target_resource,
                        is_upstream=False, is_owner=False):
    if not is_valid_resource(user_resource):
        return False
    user_list = user_resource.strip().split(":")
    target_list = target_resource.strip().split(":")
    for idx, segment in enumerate(user_list):
        if idx >= len(target_list):
            return False
        elif user_list[idx] == "*":
            continue
        elif user_list[idx] == '+'\
                and target_list[idx] != '*' and is_upstream:
            continue
        elif user_list[idx] in ('+', '-') and is_owner:
            continue
        elif target_list[idx] in ('+', '-'):
            return False
        elif user_list[idx] != target_list[idx]:
            return False
    return True


def is_resources_matched(user_resources, target_resource,
                         is_upstream=False, is_owner=False):
    for ur in user_resources:
        if is_resource_matched(ur, target_resource,
                               is_upstream=is_upstream,
                               is_owner=is_owner):
            return True
    return False


def get_perm_desc_from_string(string_desc):
    return {'action': string_desc,
            'effect': 'allow',
            'resource': '*'}


def is_perm_matched(user_perm_desc, target_perm_desc,
                    is_upstream=False, is_owner=False,
                    ignore_effect=False):
    return is_action_matched(user_perm_desc['action'],
                             target_perm_desc['action'])\
           and (user_perm_desc['effect'] == target_perm_desc['effect']
                if not ignore_effect else True)\
           and is_resource_matched(user_perm_desc['resource'],
                                    target_perm_desc['resource'],
                                    is_upstream=is_upstream,
                                    is_owner=is_owner)


def is_perm_allowed(user_perm_desc_list, target_perm_desc,
                    is_upstream=False, is_owner=False):
    if isinstance(user_perm_desc_list, dict):
        user_perm_desc_list = [user_perm_desc_list]
    allowed = False
    for perm_desc in user_perm_desc_list:
        is_matched = is_perm_matched(perm_desc, target_perm_desc,
                                     is_upstream=is_upstream, is_owner=is_owner,
                                     ignore_effect=True)
        if not is_matched:
            continue
        if perm_desc['effect'] == 'deny':
            allowed = False
            break
        else:
            allowed = True
    return allowed


def is_valid_perm_desc(perm):
    if isinstance(perm, six.string_types):
        perm = get_perm_desc_from_string(perm)
    action = perm.get('action')
    effect = perm.get('effect')
    resource = perm.get('resource')
    return action and is_valid_action(action)\
           and effect and effect.lower() in ['allow', 'deny']\
           and resource and is_valid_resource(resource)


def equ(perm1, perm2):
    return perm1['action'] == perm2['action']\
           and perm1['effect'] == perm2['effect']\
           and perm1['resource'] == perm2['resource']



# coding:utf-8
import bson
from ...backend import db


def to_dict(user, with_info=False, fields=None):
    ret = {'user_id': str(user.id),
           'created': str(user.created),
           'name': user.name,
           'soft_del': user.soft_del}
    ret['email'] = user.info.get('email')\
                   or user.info.get('orgEmail') or ''
    ret['mobile'] = user.info.get('mobile') or ''
    ret['position_id'] = user.info.get('position_id') or ''
    ret['position_name'] = user.info.get('position_name') or ''
    ret['position_tags'] = user.info.get('position_tags') or []
    ret['position_path'] = user.info.get('position_path') or []
    ret['creator_id'] = user.info.get('creator_id') or ''
    ret['creator_name'] = user.info.get('creator_name') or ''
    ret['role_names'] = user.info.get('role_names') or []
    ret['dingtalk_id'] = user.info.get('dingtalk_id') or ''
    ret['jobnumber'] = user.info.get('jobnumber') or ''
    ret['avatar'] = user.info.get('avatar') or ''

    if with_info:
        ret['info'] = user.info
    if fields is not None:
        ret = {k: v for k, v in ret.items() if k in fields}
    return ret


def list_users(q=None):
    if q:
        if isinstance(q, dict):
            user_list = db.get_list_query(db.User, **q)
        else:
            user_list = dg.get_list_query(db.User, q)
    else:
        user_list = db.get_list_query(db.User, soft_del=False)
    return True, list(map(to_dict, user_list))


def get_user_names(user_ids, soft_del=None):
    filters = {'$in': ['id', user_ids]}
    if soft_del is not None:
        filters['soft_del'] = soft_del
    user_list = db.get_list_query(db.User, **filters)
    return True, dict(map(lambda u: (str(u.id), u.name), user_list))


def get_user_info_dict(user_ids):
    user_ids = list(filter(lambda x: bson.objectid.ObjectId.is_valid(x),
                           user_ids))
    user_list = db.get_list_query(soft_del=False,
                                  **{'$in': ['id', user_ids]})
    return True, dict(map(lambda u: (str(u.id),
                          to_dict(u)),
                     user_list))


def query_users(keyword):
    pass_login = db.get_inst(db.PasswordLogin,
                             username=keyword,
                             soft_del=False)
    if pass_login:
        return list_users(q={'id': pass_login.user_id})
    q = [{'name': keyword},
         {'info.email': keyword},
         {'info.mobile': keyword}]
    if len(keyword) == 12 or len(keyword) == 24:
        q.append({'id': keyword})
    return list_users(q={'$or': q})


def query_by_info(query_type, query_value):
    q = {'soft_del': False}
    if query_type in ['email',
                      'mobile',
                      'jobnumber',
                      'position_name']:
        q["info.%s" % query_type] = query_value
    elif query_type in ['role_name',
                        'position_tag']:
            q['info.%ss' % query_type] = query_value
    return db.get_list_query(db.User, **q)


def get_by_info(query_type, query_value):
    user = query_by_info(query_type, query_value).first()
    return (True, to_dict(user) if user else None)


def enable_user(user_id):
    user = db.get_inst(db.User, id=user_id)
    if not user:
        return False, "用户不存在"
    if user.soft_del == True:
        user.soft_del = False
        user.save()
    return True, to_dict(user)


def disable_user(user_id):
    user = db.get_inst(db.User, id=user_id)
    if not user:
        return False, "用户不存在"
    if user.soft_del == False:
        user.soft_del = True
        user.save()
    return True, to_dict(user)


def update_user(user_id, **kw):
    user = db.get_inst(db.User, id=user_id)
    if not u:
        return False, "用户不存在"
    if 'name' in kw:
        u.name = kw.pop('name')
    u.info = {**u.info, **kw}
    u.save()
    return True, to_dict(u)


def get_audit_log_desc(log_entry):
    ret = {'log_id': str(log_entry.id),
           'created': log_entry.created.isoformat(),
           'user_id': log_entry.user_id,
           'editor_id': log_entry.editor_id,
           'editor_name': log_entry.editor_name}
    coll_map = {
        'user_perm': '权限信息',
        'user': '基本信息',
        'password_login': '登录信息'
    }
    field_map = {
        'name': '名字',
        'roles': '角色',
        'perms': '权限',
        'position_id': '职位',
        'granted_positions': '授权的数据范围',
        'soft_del': '有效性',
        'info': '具体信息',
        'lut': ''
    }
    msg = "修改了用户的%s" % coll_map.get(log_entry.collection_name,
                                        log_entry.collection_name)
    fields_desc = []
    for f in log_entry.fields[:3]:
        if field_map.get(f):
            fields_desc.append(field_map.get(f, f))
    if fields_desc:
        msg += ', 字段包括(但不限于)%s等' % (','.join(fields_desc))
    ret['msg'] = msg
    return ret


def get_user_audit_log(user_id, **kw):
    desc = kw.get('desc', True)
    cursor_id = kw.get('cursor_id') or None
    limit = kw.get('limit', 10)
    log_objs, last_id = db.UserAuditLog._iter(threshold=cursor_id,
                                              filters={'user_id': user_id},
                                              desc=True,
                                              limit=limit)
    ret = [get_audit_log_desc(i) for i in log_objs]
    return True, ret


def create_user(name, **info):
    user = db.User(name=name,
                   info=info)
    user.save()
    return user



# coding:utf-8
import re
import time
import uuid
import datetime
import random
import bcrypt
import six
from ...common import conf, event, PbException
from ...util.cache_util import cache_lock
from ...backend import db


try:
    import validate_email

    def is_email_valid(addr):
        return validate_email.validate_email(addr)
except ImportError as e:
    def is_email_valid(addr):
        matched = re.match(r'[^@]+@[^@]+\.[^@]+', addr)
        return True if matched else False


def to_byte(s, encoding='utf-8', errors='strict'):
    if six.PY2:
        return s.encode(encoding, errors=errors)\
               if isinstance(s, unicode) else s
    else:
        return s.encode(encoding, errors=errors)\
               if isinstance(s, six.string_types) else s


def is_password_valid(password):
    if re.match(r"^([a-zA-Z0-9.!@#$%^&*()<>{}\[\]]{4,40})$", password):
        return True
    return False


def get_password_hash(password):
    pass_hash = bcrypt.hashpw(to_byte(password), bcrypt.gensalt())
    if not isinstance(pass_hash, six.text_type):
        pass_hash = pass_hash.decode('utf-8')
    return pass_hash


def get_username(user):
    user = user if hasattr(user, 'id')\
           else db.get_inst(db.User, id=user, soft_del=False)
    if not user:
        return None
    pass_login = db.get_inst(db.PasswordLogin,
                             user_id=str(user.id), soft_del=False)
    if not pass_login:
        return None
    else:
        return pass_login.username


def get_user(username, password):
    pl = db.get_inst(db.PasswordLogin,
                     username=username, soft_del=False)
    if not pl:
        return None
    user = db.get_inst(db.User, id=pl.user_id, soft_del=False)
    if not user:
        return None
    if not bcrypt.checkpw(to_byte(password), to_byte(pl.password)):
        return None
    return user


def create_user_by_pass(username, password, **kw):
    pl = db.get_inst(db.PasswordLogin,
                     username=username, soft_del=False)
    if pl:
        return False, None, '用户已经存在'
    if not is_password_valid(password):
        return False, None, '密码格式不正确'
    if not is_email_valid(username):
        return False, None, "用户名必须为电子邮件地址"
    password_hash = get_password_hash(password)
    name = kw.get('name') or ''
    user = User(name=name)
    user.info['email'] = username
    for k, v in kw.items():
        user.info[k] = v
    user.save()
    pl = db.PasswordLogin(username=username,
                          password=password_hash,
                          user_id=str(user.id))
    pl.save()
    return True, user, ''


def attach_pass_login_to_user(user, username, password):
    if not is_password_valid(password):
        return False, None, '密码格式不正确'
    if not is_email_valid(username):
        return False, None, '用户名必须为Email'
    pl = db.get_inst(db.PasswordLogin,
                     username=username, soft_del=False)
    if pl:
        return False, None, '用户登录方式已经存在'
    password_hash = get_password_hash(password)
    user.info['email'] = username
    user.save()
    pl = db.PasswordLogin(username=username,
                          password=password_hash,
                          user_id=str(user.id))
    pl.save()
    return True, user, ''


def change_password(user, new_password):
    pl = db.get_inst(db.PasswordLogin,
                     user_id=str(user.id), soft_del=False)
    if not pl:
        return False, '用户密码不存在'
    if not is_password_valid(new_password):
        return False, '密码格式不正确'
    pl.password = get_password_hash(new_password)
    pl.save()
    return True, ''


def should_start_phone_verify(user, request):
    need_phone_verify = getattr(conf, 'NEED_PHONE_VERIFY', 'optional')
    if need_phone_verify == 'force':
        return True
    elif need_phone_verify == 'optional':
        return True if user.info.get('mobile') else False
    elif need_phone_verify == 'ip':
        ip_list = getattr(conf, 'NEED_PHONE_VERIFY_IP_WHITELIST', [])
        if request.remote_addr in ip_list:
            return False
        else:
            return True
    else:
        return False


def send_sms_code(session, method='voice'):
    password_verified = session.get('password_verified') or {}
    if not password_verified or\
            not (int(time.time()) - password_verified.get('timestamp', 0)) < 25 * 60:
        return False, "用户名/密码未验证或已失效，请返回重新验证"

    mobile = password_verified.get('mobile')
    user_id = password_verified.get('user_id')
    if not mobile:
        return False, "用户的手机号码不存在，请点击下方'重置手机号或密码'链接进行设置"
    if not user_id:
        return False, "用户信息错误，请重试"

    sms_code = session.get('sms_code')
    if not sms_code or\
            (sms_code and sms_code.get('user_id') != user_id):
        code = ''.join([random.choice('0123456789') for i in range(6)])
        sms_code = {'code': code,
                    'user_id': user_id,
                    'timestamp': int(time.time())}
    elif int(time.time()) - sms_code.get('timestamp') > 20 * 60:
        code = ''.join([random.choice('0123456789') for i in range(6)])
        sms_code['code'] = code
    elif int(time.time()) - sms_code.get('timestamp') < 60:
        return False, "验证码发送间隔时间太短"
    else:
        code = sms_code['code']
    if method in ['voice', 'sms']:
        try:
            event.default.send(method,
                               **{"mobile": mobile,
                                  "code": code})
        except PbException as e:
            return False, e.args[0]
    else:
        return False, "不支持的验证发送方式"

    sms_code['timestamp'] = int(time.time())
    session['sms_code'] = sms_code
    return True, ''


def verify_sms_code(session, code):
    password_verified = session.get('password_verified')
    sms_code = session.get('sms_code')
    if not (sms_code and password_verified):
        return False, ''
    if sms_code.get('user_id') != password_verified.get('user_id'):
        return False, ''
    if int(time.time()) - sms_code.get('timestamp', 0) > 20 * 60\
            or int(time.time()) - password_verified.get('timestamp', 0) > 25 * 60:
        return False, ''
    if not (code and sms_code.get('code')):
        return False, ''
    ret = sms_code.get('code') == code
    if not ret:
        return False, ''
    user = db.get_inst(db.User,
                       id=sms_code['user_id'])
    if not user:
        return False, ''
    return True, user

RESET_TEMPLATE = '''您好，
您发起的密码或者安全手机重置操作，请访问 %s%s 。
如果您没有手动发起这个操作，请直接忽略。
'''


def get_reset_addr_base(request):
    referrer = request.referrer or ''
    if not referrer:
        return "/static/gbm/#/resetpwd?code="
    idx = referrer.find('#')

    if idx != -1:
        return referrer[:idx] + '#/resetpwd?code='
    else:
        return referrer + '#/resetpwd?code='


def send_reset_email(email_addr, reset_addr_base):
    pl = db.get_inst(db.PasswordLogin,
                     username=email_addr, soft_del=False)
    if not pl:
        return False, '用户信息错误'
    user = db.get_inst(db.User,
                       id=pl.user_id, soft_del=False)
    if not user:
        return False, '用户信息错误'
    curr = int(time.time())
    record_count = db.get_list_query(db.ResetPasswordRecord,
                                     user_id=str(user.id),
                                     **{'$gt': ['timestamp', (curr - 86400)],
                                        '$ne': ['state', 'used']}).count()
    if record_count >= 50:
        return False, '24小时内重置次数超过阈值'
    code = uuid.uuid1().hex + uuid.uuid4().hex
    record = db.ResetPasswordRecord(code=code,
                                    user_id=str(user.id),
                                    state='created')
    record.save()
    event.default.send("reset-email",
                       **{"email_addr": email_addr,
                          "reset_addr_base": reset_addr_base,
                          "code": code})
    return True, record


def verify_reset_code(code, **kw):
    with cache_lock(code):
        curr = int(time.time())
        record = db.get_inst(db.ResetPasswordRecord,
                             code=code,
                             state='created',
                             **{'$gt': ['timestamp', (curr - 86400)]})
        if record:
            record.query_times += 1
            if 'verifies' not in record.info:
                record.info['verifies'] = []
            entry = {'timestamp': curr,
                     'date': str(datetime.datetime.now())}
            entry.update(kw)
            record.info['verifies'].append(entry)
            record.save()
        if record and record.query_times < 5:
            return True, record
        return False, '重置记录不存在或者已失效'


def reset_record(code, password='', mobile=''):
    with cache_lock(code):
        curr = int(time.time())
        record = db.get_inst(db.ResetPasswordRecord,
                             code=code,
                             state='created',
                             **{'$gt': ['timestamp', (curr - 86400)]})
        if not record:
            return False, '重置记录不存在或者已失效'
        if record.query_times >= 5 or not (password or mobile):
            record.state = 'invalid'
            record.save()
            return False, '重置记录不存在或者已失效'
        user = db.get_inst(db.User,
                           id=record.user_id,
                           soft_del=False)
        pl = db.get_inst(db.PasswordLogin,
                         user_id=record.user_id,
                         soft_del=False)
        if not user and pl:
            record.state = 'invalid'
            record.save()
            return False, '重置记录不存在或者已失效'
        if mobile:
            if not re.match(r"^[1][3456789][\d]{9}$", mobile):
                return False, '手机号码格式不正确'
            user.info['mobile'] = mobile
            record.info['mobile'] = True
            user.save()
        if password:
            record.info['password'] = True
            change_password(user, password)
        record.state = 'used'
        record.save()
        return True, record



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['perm_broker',
 'perm_broker.auth',
 'perm_broker.auth.ops',
 'perm_broker.auth.views',
 'perm_broker.backend',
 'perm_broker.backend.mongodb',
 'perm_broker.backend.mssql',
 'perm_broker.backend.mysql',
 'perm_broker.common',
 'perm_broker.perm',
 'perm_broker.perm.ops',
 'perm_broker.perm.views',
 'perm_broker.user',
 'perm_broker.user.ops',
 'perm_broker.user.views',
 'perm_broker.util']

package_data = \
{'': ['*'], 'perm_broker': ['templates/*']}

setup_kwargs = {
    'name': 'perm-broker',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'Gongziting Tech Ltd.',
    'author_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
'''
Config module for pauli, also containing default config values.
'''

# Debug Flag
DEBUG = False

# Default session lifetime
PAULI_SESSION_LIFETIME = 86400

# The cookie name for session id
SESSION_COOKIE_NAME = 'sessionid'

# Automatically create user from Dingtalk signin
DINGTALK_AUTO_CREATE_USER = False

# The primary fields for checking duplication of a Dingtalk user
USER_DINGTALK_PRIMARY_FIELDS = ["jobnumber"]

# The fields that should be synchronized to the field of a user
USER_INFO_DINGTALK_FIELDS = [
    "name",
    "email",
    "orgEmail",
    "mobile",
    "jobnumber",
    "avatar"
]

# Overwrite user's info from dingtalk as the default behavior
USER_INFO_DINGTALK_DEFAULT_OVERWRITE = False

# Enabled login methods
LOGIN_METHODS = ['password']



# -*- coding: utf-8 -*-
'''
Provide basic utilities for api.
'''
import json
import flask


STATUS_CODES = {
    'E_SUCC': (0, '成功'),
    'E_PARAM': (1, '参数错误'),
    'E_INTER': (2, '程序内部错误'),
    'E_EXTERNAL': (3, '外部接口错误'),
    'E_TIMEOUT': (4, '第三方接口超时'),
    'E_RESRC': (5, '接口不存在'),
    'E_AUTH': (6, '鉴权失败'),
    'E_FORBIDDEN': (7, '访问被禁止'),
    'E_RESOURCE_NOT_FIND': (8, '资源不存在或已删除'),
    'E_USER_NOT_EXIST_IN_CRM': (1024, 'CRM系统中不存在该用户'),
    'E_USER_LOCKED': (1000, '用户被锁定'),
    'E_USER_EXIST': (1001, '该手机号已被注册'),
    'E_USER_NOT_EXIST': (1002, '用户不存在, 请检查手机号码是否正确，或者前往注册'),
    'E_USER_LOGIN': (1003, '用户已登录'),
    'E_USER_NOT_LOGIN': (1004, '用户未登录'),
    'E_USER_NAME_OR_PWD_ERROR': (1005, '手机号码与密码不匹配'),
    'E_USER_VERIFY_CODE_LOCKED': (1006, '请不要在60秒内连续发送验证码'),
    'E_USER_VERIFY_CODE_ERROR': (1007, '验证码错误，请输入正确的验证码'),
    'E_USER_LOGIN_PWD_NOT_SET': (1008, '账户未设置过密码，请使用验证码登录'),
    'E_USER_HEAD_UPLOAD_ERROR': (1010, '头像上传失败'),
    'E_USER_AUTH_PWD_VERIFY_RETRY': (1011, '密码验证错误'),
    'E_USER_AUTH_PWD_VERIFY_ERROR': (1012, '密码验证失败，您将退出登录'),
    'E_USER_PHONE_NUMBER_ERROR': (1013, '手机号格式错误'),
    'E_USER_IDENTITY_CARD_ERROR': (1014, '身份证号码不合法'),
    'E_USER_BANK_CARD_ERROR': (1015, '银行卡号不合法'),
    'E_USER_MODIFY_RISK_ERROR': (1016, '提交风险承受能力失败'),
    'E_USER_BUS_TYPE_INVALID': (1017, '验证业务类型不匹配'),
    'E_USER_AUTH_TOKEN_INVALID': (1018, '鉴权令牌已失效，请重新发送验证码'),
    'E_USER_AUTH_TOKEN_ERROR': (1019, '鉴权令牌校验失败'),
    'E_USER_AUTH_NOT_EXIST': (1020, '手机号码不存在，无法获取验证码'),
    'E_USER_AUTH_PWD_VERIFY_INVALID': (1021, '密码格式不正确'),
    'E_USER_ORG_LOGIN_PWD_ERROR': (1022, '原登录密码错误'),
    'USER_NEED_SMS_AUTH': (1023, '需要进行短信验证')
}


def send_json_result(code, result=None, msg=None):
    sc_code = 'E_%s' % code if isinstance(code, str) else code
    status = STATUS_CODES.get(sc_code)
    ret = json.dumps({'code': status[0],
                      'result': {} if result is None else result,
                      'msg': msg or status[1]})
    resp = flask.Response(ret)
    resp.content_type = 'application/json'
    return resp



# -*- coding:utf-8 -*-
'''
Basic root blueprint entry point.
'''
import os
import flask
from . import conf
from . import event
from .exceptions import PbException


pb_root = flask.Blueprint("pb_root", __name__,
                          template_folder=os.path.abspath(os.path.dirname(__file__) + '/../templates'))

__ALL__ = ['pb_root', 'conf', 'event', 'PbException']



# -*- coding: utf-8 -*-
import logging
import time
import json
import blinker

logger = logging.getLogger(__name__)

default = blinker.signal("pauli-default")


@default.connect
def event_log(*sub, **message):
    logger.info(json.dumps({
        "sender": sub,
        "message": message,
        "timestamp": int(time.time())
    }))



# coding:utf-8
import six
from ...backend import db
from . import perm_base


def get_or_create_user_perm(user_id):
    user_perm = db.get_inst(db.UserPerm, user_id=user_id, soft_del=False)
    if not user_perm:
        user = db.get_inst(db.User, id=user_id, soft_del=False)
        if not user:
            return None
        user_perm = db.UserPerm(user_id=user_id)
        user_perm.save()
    return user_perm


def add_perm(perm, user_id=None, role_id=None):
    if isinstance(perm, six.string_types):
        perm = perm_base.get_perm_desc_from_string(perm)
    if not perm_base.is_valid_perm_desc(perm):
        return False, "错误的权限描述"
    if user_id:
        user_perm = get_or_create_user_perm(user_id)
        if not user_perm:
            return False, "目标用户不存在"
        if any(map(lambda x: perm_base.equ(perm, x), user_perm.perms)):
            return False, "重复的权限描述"
        user_perm.perms = [*user_perm.perms, perm]
        user_perm.save()
        return True, user_perm
    elif role_id:
        role = db.get_inst(db.RoleDesc, id=role_id)
        if not role:
            return False, "目标角色不存在"
        if any(map(lambda x: perm_base.equ(perm, x), role.perms)):
            return False, "重复的权限描述"
        role.perms = [*role.perms, perm]
        role.save()
        return True, role
    else:
        return False, "用户或者角色id未提供"


def update_perms(perms, user_id=None, role_id=None):
    parsed_perms = []
    for perm in perms:
        if isinstance(perm, six.string_types):
            perm = perm_base.get_perm_desc_from_string(perm)
        if not perm_base.is_valid_perm_desc(perm):
            return False, "错误的权限描述 %s" % perm
        parsed_perms.append(perm)
    if user_id:
        user_perm = get_or_create_user_perm(user_id)
        if not user_perm:
            return False, "目标用户不存在"
        user_perm.perms = parsed_perms
        user_perm.save()
        return True, user_perm
    elif role_id:
        role = db.get_inst(db.RoleDesc, id=role_id)
        if not role:
            return False, "目标角色不存在"
        user_perm.perms = parsed_perms
        role.save()
        return True, role
    else:
        return False, "用户或者角色id未提供"


def remove_perm(perm, user_id=None, role_id=None):
    if isinstance(perm, six.string_types):
        perm = perm_base.get_perm_desc_from_string(perm)
    if not perm_base.is_valid_perm_desc(perm):
        return False, "错误的权限描述"
    if user_id:
        user_perm = get_or_create_user_perm(user_id)
        if not user_perm:
            return False, "目标用户不存在"
        user_perm.perms = list(filter(lambda x: not perm_base.equ(perm, x),
                                      user_perm.perms))
        user_perm.save()
        return True, user_perm
    elif role_id:
        role = db.get_inst(db.RoleDesc, id=role_id, soft_del=False)
        if not role:
            return False, "目标角色不存在"
        role.perms = list(filter(lambda x: not perm_base.equ(perm, x),
                                 role.perms))
        role.save()
        return True, role
    else:
        raise Exception("Neither user_id and role_id is available.")


def get_user_perm_list(user_id, exclude_role=False):
    '''
    获取用户的权限列表.
    user_id: str, 用户的id
    exclude_role: boolean, 是否要排除掉用户的角色中的权限
    '''
    user_perm = db.get_inst(db.UserPerm, user_id=user_id, soft_del=False)
    if not user_perm:
        return []
    else:
        perms = []
        perms.extend(user_perm.perms)
        if (not exclude_role) and user_perm.roles:
            roles = db.get_list_query(db.RoleDesc,
                                      **{'$in': ['id', user_perm.roles],
                                         'soft_del': False})
            for role in roles:
                perms.extend(role.perms)
        return perms


def has_perm(user_id, target_perm_desc,
             is_upstream=False, is_owner=False, perm_list=None):
    '''
    检查用户是否拥有某个指定权限。
    perm_list: list, optional, 如果提供此参数，则不每次调用时根据user_id获取
               用户的权限列表。
    '''
    if isinstance(target_perm_desc, six.string_types):
        target_perm_desc = perm_base\
                .get_perm_desc_from_string(target_perm_desc)
    perm_list = perm_list or get_user_perm_list(user_id)
    if perm_base.is_perm_allowed(perm_list, target_perm_desc,
                                 is_upstream=is_upstream, is_owner=is_owner):
        return True
    #for i in perm_list:
    #    if perm_base.is_perm_matched(i, target_perm_desc,
    #            is_upstream=is_upstream, is_owner=is_owner):
    #        return True
    return False


def get_user_roles(user_id):
    '''
    Return the role objects.
    '''
    user_perm = db.get_inst(db.UserPerm, user_id=user_id, soft_del=False)
    if user_perm and user_perm.roles:
        roles = db.get_list_query(db.RoleDesc,
                                  **{'$id': {'id': user_perm.roles},
                                     'soft_del': False})
        return roles
    return []


def refresh_user_info_roles(user_id, roles):
    user = db.get_inst(db.User, id=user_id)
    if user:
        role_names = []
        if roles:
            role_names = [role_desc.name for role_desc in\
                          db.get_list_query(db.RoleDesc,
                                            {'$in': ['id', roles],
                                             'soft_del': False})]
        user.info['role_names'] = role_names
        user.info = {**user.info, **{'role_names': role_names}}
        user.save()
        return True
    else:
        return False


def update_user_roles(user_id, role_ids):
    user_perm = get_or_create_user_perm(user_id)
    if not user_perm:
        return False, "用户不存在或已删除"
    role_ids = list(set(role_ids))
    role_count = db.get_list_query(db.RoleDesc,
                                   **{'$in': ['id', role_ids],
                                      'soft_del': False}).count()
    if len(role_ids) != role_count:
        return False, "存在无效的用户角色"
    user_perm.roles = role_ids
    user_perm.save()
    refresh_user_info_roles(user_perm.user_id, user_perm.roles)
    return True, user_perm


def add_role_to_user(role_id, user_id):
    user_perm = get_or_create_user_perm(user_id)
    if not user_perm:
        return False, "用户不存在或已删除"
    role = db.get_inst(db.RoleDesc, id=role_id, soft_del=False)
    if not role:
        return False, "角色不存在或已经删除"
    if not role_id in user_perm.roles:
        user_perm.roles = [*user_perm.roles, role_id]
        user_perm.save()
        refresh_user_info_roles(user_perm.user_id, user_perm.roles)
    return True, user_perm


def remove_role_from_user(role_id, user_id):
    user_perm = get_or_create_user_perm(user_id)
    if not user_perm:
        return False, "用户不存在或已删除"
    user_perm.roles = list(filter(lambda x: x != role_id,
                                  user_perm.roles))
    user_perm.save()
    refresh_user_info_roles(user_perm.user_id, user_perm.roles)
    return True, user_perm


def get_all_roles(soft_del=False):
    roles = db.get_list_query(db.RoleDesc, soft_del=soft_del)
    return list(roles)


def get_role_by_foreign_key(foreign_key):
    if not foreign_key:
        return None
    role_desc = RoleDesc.objects(foreign_key=foreign_key,
                                 soft_del=False).first()
    return role_desc


def create_role(name=None, foreign_key=None):
    if not name:
        return False, "角色名未提供"
    role_desc = db.get_inst(db.RoleDesc, name=name, soft_del=False)
    if role_desc:
        return False, "角色名已经存在"
    role_desc = get_role_by_foreign_key(foreign_key)
    if role_desc:
        return False, "外键已经存在"
    role_desc = db.RoleDesc(name=name, foreign_key=foreign_key)
    role_desc.save()
    return get_role_info(role_desc)


def remove_role(role_id):
    if not role_id:
        return False, "角色id未提供"
    role = db.get_inst(db.RoleDesc, id=role_id)
    if not role:
        return False, "角色不存在"
    role.soft_del = True
    role.save()
    return get_role_info(role)


def update_role(role_id, name=None):
    if not role_id:
        return False, "角色id未提供"
    if not name:
        return False, "名字未提供"
    duplicated_role = db.get_inst(db.RoleDesc,
                                  **{'soft_del': False,
                                     'name': name,
                                     '$ne': ['id', role_id]})
    if duplicated_role:
        return False, "同名角色已经存在"
    role = db.get_inst(db.RoleDesc,
                       id=role_id, soft_del=False)
    if not role:
        return False, "角色不存在"
    role.name = name
    role.save()
    return get_role_info(role)


def get_role_info(role=None, role_id=None, role_name=None):
    if not role:
        if role_id:
            role = db.get_inst(db.RoleDesc, id=role_id)
        elif role_name:
            role = db.get_inst(db.RoleDesc, name=role_name, soft_del=False)
    if not role:
        return False, "角色不存在"
    ret = {'id': str(role.id),
           'name': str(role.name),
           'perms': role.perms,
           'granted_positions': role.granted_positions,
           'foreign_key': role.foreign_key,
           'created': str(role.created),
           'soft_del': role.soft_del,
           'lut': str(role.lut)}
    return True, ret


def get_perm_triples(actions, user_id):
    ret = {}
    perm_list = get_user_perm_list(user_id)
    for action in actions:
        ret.setdefault(action, {})
        ret[action]['*'] = has_perm(user_id, action, perm_list=perm_list)
        ret[action]['+'] = has_perm(user_id, {'action': action, 'effect': 'allow', 'resource': '+'},
                           is_upstream=True, perm_list=perm_list)
        ret[action]['-'] = has_perm(user_id, {'action': action, 'effect': 'allow', 'resource': '-'},
                           is_owner=True, perm_list=perm_list)
    return True, ret
